@@ -21,3 +21,20 @@ __version__ = get_versions()['version']
 del get_versions
 
 version = __version__
+
+# Set up PyXll, if available
+try:
+    from .xl_interface.instrument_generation import install_hook
+    install_hook()
+except ModuleNotFoundError:
+    pass
+
+# Jupyter needs nest_asyncio to avoid event loop issues
+try:
+    from IPython import get_ipython
+    ipython = get_ipython()
+    if ipython and 'IPKernelApp' in get_ipython().config:
+        import nest_asyncio
+        nest_asyncio.apply()
+except ImportError:
+    pass
\ No newline at end of file
