@@ -14,28 +14,27 @@ specific language governing permissions and limitations
 under the License.
 """
 import datetime as dt
+import logging
 from enum import Enum
 from itertools import chain
-from typing import Iterable, List, Optional, Tuple, Union
-
-import logging
+from typing import Iterable, List, Optional, Tuple, Union, Dict
 from urllib.parse import urlencode
 
 import cachetools
 import numpy
 import pandas as pd
 from cachetools import TTLCache
+from gs_quant.target.common import FieldFilterMap, XRef, MarketDataVendor, PricingLocation
+from gs_quant.target.coordinates import MDAPIDataBatchResponse, MDAPIDataQuery, MDAPIDataQueryResponse, MDAPIQueryField
+from gs_quant.target.data import DataQuery, DataQueryResponse
+from gs_quant.target.data import DataSetEntity
 
-from gs_quant.base import Base
 from gs_quant.api.data import DataApi
-from gs_quant.data.core import DataContext
+from gs_quant.base import Base
+from gs_quant.data.core import DataContext, DataFrequency
 from gs_quant.errors import MqValueError
 from gs_quant.markets import MarketDataCoordinate
 from gs_quant.session import GsSession
-from gs_quant.target.common import FieldFilterMap, XRef, MarketDataVendor, PricingLocation
-from gs_quant.target.data import DataQuery, DataQueryResponse, MDAPIDataBatchResponse, MDAPIDataQuery, \
-    MDAPIDataQueryResponse, MDAPIQueryField
-from gs_quant.target.data import DataSetEntity
 from .assets import GsAssetApi, GsIdType
 from ...target.assets import EntityQuery
 
@@ -45,11 +44,14 @@ _logger = logging.getLogger(__name__)
 class QueryType(Enum):
     IMPLIED_VOLATILITY = "Implied Volatility"
     IMPLIED_CORRELATION = "Implied Correlation"
+    REALIZED_CORRELATION = "Realized Correlation"
     AVERAGE_IMPLIED_VOLATILITY = "Average Implied Volatility"
     AVERAGE_IMPLIED_VARIANCE = "Average Implied Variance"
     AVERAGE_REALIZED_VOLATILITY = "Average Realized Volatility"
     SWAP_RATE = "Swap Rate"
     SWAP_ANNUITY = "Swap Annuity"
+    SWAPTION_PREMIUM = "Swaption Premium"
+    SWAPTION_ANNUITY = "Swaption Annuity"
     BASIS_SWAP_RATE = "Basis Swap Rate"
     SWAPTION_VOL = "Swaption Vol"
     MIDCURVE_VOL = "Midcurve Vol"
@@ -61,6 +63,8 @@ class QueryType(Enum):
     ATM_FWD_RATE = "Atm Fwd Rate"
     BASIS = "Basis"
     VAR_SWAP = "Var Swap"
+    MIDCURVE_PREMIUM = "Midcurve Premium"
+    MIDCURVE_ANNUITY = "Midcurve Annuity"
     MIDCURVE_ATM_FWD_RATE = "Midcurve Atm Fwd Rate"
     CAP_FLOOR_ATM_FWD_RATE = "Cap Floor Atm Fwd Rate"
     SPREAD_OPTION_ATM_FWD_RATE = "Spread Option Atm Fwd Rate"
@@ -85,6 +89,16 @@ class QueryType(Enum):
     G_REGIONAL_SCORE = "G Regional Score"
     G_REGIONAL_PERCENTILE = "G Regional Percentile"
     ES_DISCLOSURE_PERCENTAGE = "ES Disclosure Percentage"
+    RATING = "Rating"
+    CONVICTION_LIST = "Conviction List"
+    GIR_GSDEER_GSFEER = "Gir Gsdeer Gsfeer"
+    GIR_FX_FORECAST = "Gir Fx Forecast"
+    GROWTH_SCORE = "Growth Score"
+    FINANCIAL_RETURNS_SCORE = "Financial Returns Score"
+    MULTIPLE_SCORE = "Multiple Score"
+    INTEGRATED_SCORE = "Integrated Score"
+    GIR_COMMODITIES_FORECAST = "Gir Commodities Forecast"
+    FORECAST_VALUE = "Forecast Value"
 
 
 class GsDataApi(DataApi):
@@ -96,12 +110,11 @@ class GsDataApi(DataApi):
 
     @classmethod
     def query_data(cls, query: Union[DataQuery, MDAPIDataQuery], dataset_id: str = None,
-                   asset_id_type: Union[GsIdType, str] = None)\
+                   asset_id_type: Union[GsIdType, str] = None) \
             -> Union[MDAPIDataBatchResponse, DataQueryResponse, tuple]:
         if isinstance(query, MDAPIDataQuery) and query.market_data_coordinates:
             # Don't use MDAPIDataBatchResponse for now - it doesn't handle quoting style correctly
-            results: Union[MDAPIDataBatchResponse, dict] = GsSession.current._post(
-                '/data/coordinates/query', payload=query)
+            results: Union[MDAPIDataBatchResponse, dict] = cls.execute_query('coordinates', query)
             if isinstance(results, dict):
                 return results.get('responses', ())
             else:
@@ -133,12 +146,9 @@ class GsDataApi(DataApi):
                     setattr(query.where, xref_type, None)
                     query.where.assetId = [asset_id_map[x] for x in xref_values]
 
-        results: Union[DataQueryResponse, dict] = GsSession.current._post('/data/{}/query'.format(dataset_id),
-                                                                          payload=query)
-        if isinstance(results, dict):
-            results = results.get('data', ())
-        else:
-            results = results.data if results.data is not None else ()
+        response: Union[DataQueryResponse, dict] = cls.execute_query(dataset_id, query)
+
+        results = cls.get_results(dataset_id, response, query)
 
         if asset_id_type not in {GsIdType.id, None}:
             asset_ids = tuple(set(filter(None, (r.get('assetId') for r in results))))
@@ -153,6 +163,31 @@ class GsDataApi(DataApi):
 
         return results
 
+    @staticmethod
+    def execute_query(dataset_id: str, query: Union[DataQuery, MDAPIDataQuery]):
+        return GsSession.current._post('/data/{}/query'.format(dataset_id), payload=query)
+
+    @staticmethod
+    def get_results(dataset_id: str, response: Union[DataQueryResponse, dict], query: DataQuery) -> list:
+        if isinstance(response, dict):
+            total_pages = response.get('totalPages')
+            results = response.get('data', ())
+        else:
+            total_pages = response.total_pages if response.total_pages is not None else 0
+            results = response.data if response.data is not None else ()
+
+        if total_pages:
+            if query.page is None:
+                query.page = total_pages - 1
+                results = results + GsDataApi.get_results(dataset_id, GsDataApi.execute_query(dataset_id, query), query)
+            elif query.page - 1 > 0:
+                query.page -= 1
+                results = results + GsDataApi.get_results(dataset_id, GsDataApi.execute_query(dataset_id, query), query)
+            else:
+                return results
+
+        return results
+
     @classmethod
     def last_data(cls, query: Union[DataQuery, MDAPIDataQuery], dataset_id: str = None) -> Union[list, tuple]:
         if getattr(query, 'marketDataCoordinates', None):
@@ -182,7 +217,8 @@ class GsDataApi(DataApi):
             scroll_id: Optional[str] = None,
             limit: int = None,
             offset: int = None,
-            fields: List[str] = None
+            fields: List[str] = None,
+            include_history: bool = False
     ) -> List[dict]:
         params = {}
         if scroll:
@@ -201,6 +237,9 @@ class GsDataApi(DataApi):
         if fields:
             params['fields'] = fields
 
+        if include_history:
+            params['includeHistory'] = 'true'
+
         body = GsSession.current._get('/data/{}/coverage'.format(dataset_id), payload=params)
         results = body['results']
         if len(results) > 0 and 'scrollId' in body:
@@ -292,7 +331,7 @@ class GsDataApi(DataApi):
             raise NotImplementedError('Unsupported return type')
 
     @staticmethod
-    def build_market_data_query(asset_ids: List[str], query_type: QueryType, where: Union[FieldFilterMap, dict] = None,
+    def build_market_data_query(asset_ids: List[str], query_type: QueryType, where: Union[FieldFilterMap, Dict] = None,
                                 source: Union[str] = None, real_time: bool = False):
         inner = {
             'assetIds': asset_ids,
@@ -314,6 +353,46 @@ class GsDataApi(DataApi):
             'queries': [inner]
         }
 
+    @classmethod
+    def get_data_providers(cls, entity_id: str) -> Dict:
+        """Return daily and real-time data providers
+
+        :param entity_id: identifier of entity i.e. asset, country, subdivision
+        :return: dictionary of available data providers
+
+        ** Usage **
+
+        Return a dictionary containing a set of dataset providers for each available data field.
+        For each field will return a dict of daily and real-time dataset providers where available.
+        """
+
+        GsSession.current: GsSession
+        body = GsSession.current._get(f'/data/measures/{entity_id}/availability')
+        if 'errorMessages' in body:
+            raise MqValueError(f"data availablity request {body['requestId']} failed: {body.get('errorMessages', '')}")
+        if 'data' not in body:
+            providers = {}
+        else:
+            providers = {}
+
+            all_data_mappings = sorted(body['data'], key=lambda x: x['rank'], reverse=True)
+
+            for source in all_data_mappings:
+
+                freq = source.get('frequency', 'End Of Day')
+                dataset_field = source.get('datasetField', '')
+                rank = source.get('rank')
+
+                providers.setdefault(dataset_field, {})
+
+                if rank:
+                    if freq == 'End Of Day':
+                        providers[dataset_field][DataFrequency.DAILY] = source['datasetId']
+                    elif freq == 'Real Time':
+                        providers[dataset_field][DataFrequency.REAL_TIME] = source['datasetId']
+
+        return providers
+
     @classmethod
     def get_market_data(cls, query) -> pd.DataFrame:
         GsSession.current: GsSession
@@ -333,9 +412,9 @@ class GsDataApi(DataApi):
     @classmethod
     def __normalise_coordinate_data(
             cls,
-            data: Iterable[Union[MDAPIDataQueryResponse, dict]],
+            data: Iterable[Union[MDAPIDataQueryResponse, Dict]],
             fields: Optional[Tuple[MDAPIQueryField, ...]] = None
-    ) -> Iterable[Iterable[dict]]:
+    ) -> Iterable[Iterable[Dict]]:
         ret = []
         for response in data:
             coord_data = []
@@ -362,11 +441,13 @@ class GsDataApi(DataApi):
     @classmethod
     def __df_from_coordinate_data(
             cls,
-            data: Iterable[dict]
+            data: Iterable[Dict],
+            *,
+            use_datetime_index: Optional[bool] = True
     ) -> pd.DataFrame:
         df = cls._sort_coordinate_data(pd.DataFrame.from_records(data))
         index_field = next((f for f in ('time', 'date') if f in df.columns), None)
-        if index_field:
+        if index_field and use_datetime_index:
             df = df.set_index(pd.DatetimeIndex(df.loc[:, index_field].values))
 
         return df
@@ -410,7 +491,7 @@ class GsDataApi(DataApi):
             vendor: MarketDataVendor = MarketDataVendor.Goldman_Sachs,
             as_dataframe: bool = False,
             pricing_location: Optional[PricingLocation] = None
-    ) -> Union[dict, pd.DataFrame]:
+    ) -> Union[Dict, pd.DataFrame]:
         """
         Get last value of coordinates data
 
@@ -428,7 +509,6 @@ class GsDataApi(DataApi):
         """
         market_data_coordinates = tuple(cls._coordinate_from_str(coord) if isinstance(coord, str) else coord
                                         for coord in coordinates)
-        ret = {coordinate: None for coordinate in market_data_coordinates}
         query = cls.build_query(
             end=as_of,
             market_data_coordinates=market_data_coordinates,
@@ -438,17 +518,24 @@ class GsDataApi(DataApi):
 
         data = cls.last_data(query)
 
+        if not as_dataframe:
+            ret = {coordinate: None for coordinate in market_data_coordinates}
+            for idx, row in enumerate(cls.__normalise_coordinate_data(data)):
+                try:
+                    ret[market_data_coordinates[idx]] = row[0]['value']
+                except IndexError:
+                    ret[market_data_coordinates[idx]] = None
+            return ret
+
+        ret = []
         for idx, row in enumerate(cls.__normalise_coordinate_data(data)):
+            coordinate_as_dict = market_data_coordinates[idx].as_dict(as_camel_case=True)
             try:
-                ret[market_data_coordinates[idx]] = row[0]['value']
+                ret.append(dict(chain(coordinate_as_dict.items(),
+                                      (('value', row[0]['value']), ('time', row[0]['time'])))))
             except IndexError:
-                ret[market_data_coordinates[idx]] = None
-
-        if as_dataframe:
-            data = [dict(chain(c.as_dict(as_camel_case=True).items(), (('value', v),))) for c, v in ret.items()]
-            return cls.__df_from_coordinate_data(data)
-
-        return ret
+                ret.append(dict(chain(coordinate_as_dict.items(), (('value', None), ('time', None)))))
+        return cls.__df_from_coordinate_data(ret, use_datetime_index=False)
 
     @classmethod
     def coordinates_data(
@@ -460,6 +547,7 @@ class GsDataApi(DataApi):
             as_multiple_dataframes: bool = False,
             pricing_location: Optional[PricingLocation] = None,
             fields: Optional[Tuple[MDAPIQueryField, ...]] = None,
+            **kwargs
     ) -> Union[pd.DataFrame, Tuple[pd.DataFrame]]:
         """
         Get coordinates data
@@ -471,6 +559,7 @@ class GsDataApi(DataApi):
         :param as_multiple_dataframes: whether to return the result as one or multiple Dataframe(s)
         :param pricing_location: the location where close data has been recorded (not used for real-time query)
         :param fields: value fields to return
+        :param kwargs: Extra query arguments
         :return: Dataframe(s) of the returned data
 
         **Examples**
@@ -486,7 +575,8 @@ class GsDataApi(DataApi):
             start=start,
             end=end,
             pricing_location=pricing_location,
-            fields=fields
+            fields=fields,
+            **kwargs
         )
 
         results = cls.__normalise_coordinate_data(cls.query_data(query), fields=fields)
@@ -503,7 +593,8 @@ class GsDataApi(DataApi):
             start: Union[dt.datetime, dt.date] = None,
             end: Union[dt.datetime, dt.date] = None,
             vendor: MarketDataVendor = MarketDataVendor.Goldman_Sachs,
-            pricing_location: Optional[PricingLocation] = None
+            pricing_location: Optional[PricingLocation] = None,
+            **kwargs
     ) -> Union[pd.Series, Tuple[pd.Series]]:
         """
         Get coordinates data series
@@ -513,6 +604,7 @@ class GsDataApi(DataApi):
         :param end: end date or time
         :param vendor: data vendor
         :param pricing_location: the location where close data has been recorded (not used for real-time query)
+        :param kwargs: Extra query arguments
         :return: Series of the returned data
 
         **Examples**
@@ -526,7 +618,9 @@ class GsDataApi(DataApi):
             end=end,
             pricing_location=pricing_location,
             vendor=vendor,
-            as_multiple_dataframes=True)
+            as_multiple_dataframes=True,
+            **kwargs
+        )
 
         ret = tuple(pd.Series() if df.empty else pd.Series(index=df.index, data=df.value.values) for df in dfs)
         if isinstance(coordinates, (MarketDataCoordinate, str)):
@@ -549,18 +643,21 @@ class GsDataApi(DataApi):
         raise RuntimeError(f"Unable to get Dataset schema for {dataset_id}")
 
     @classmethod
-    def construct_dataframe_with_types(cls, dataset_id: str, data: Union[Base, list, tuple]) -> pd.DataFrame:
+    def construct_dataframe_with_types(cls, dataset_id: str, data: Union[Base, List, Tuple]) -> pd.DataFrame:
         """
         Constructs a dataframe with correct date types.
+        :param dataset_id: id of the dataset
         :param data: data to convert with correct types
         :return: dataframe with correct types
         """
         if len(data):
             dataset_types = cls.get_types(dataset_id)
-            df = pd.DataFrame(data)
+
+            df = pd.DataFrame(data, columns=dataset_types)
 
             for field_name, type_name in dataset_types.items():
-                if df.get(field_name) is not None and type_name in ('date', 'date-time'):
+                if df.get(field_name) is not None and type_name in ('date', 'date-time') and \
+                        len(df.get(field_name).value_counts()) > 0:
                     df = df.astype({field_name: numpy.datetime64})
 
             field_names = dataset_types.keys()
