@@ -20,22 +20,21 @@ import json
 import logging
 import math
 import msgpack
-from queue import Queue
 import time
 from typing import Iterable, Optional, Union
-from websockets import ConnectionClosedError, client
+from websockets import ConnectionClosedError
 
 from gs_quant.api.risk import RiskApi
 from gs_quant.risk import RiskRequest
 from gs_quant.session import GsSession
-from gs_quant.target.risk import APEXOptimizationRequest
+from gs_quant.target.risk import OptimizationRequest
 
 _logger = logging.getLogger(__name__)
 
 
 class GsRiskApi(RiskApi):
 
-    USE_MSGPACK = True
+    USE_MSGPACK = False
 
     @classmethod
     def calc_multi(cls, requests: Iterable[RiskRequest]) -> dict:
@@ -67,59 +66,98 @@ class GsRiskApi(RiskApi):
         return '/risk{}/calculate{}'.format('-internal' if is_internal else '', '/bulk' if is_bulk else '')
 
     @classmethod
-    async def get_results(cls, responses: asyncio.Queue, results: Queue, timeout: Optional[int] = None):
-        async def send_to_websocket(ws: client):
-            while True:
-                items = await cls.drain_queue_async(responses)
-                if not items:
-                    break
+    async def get_results(cls, responses: asyncio.Queue, results: asyncio.Queue, timeout: Optional[int] = None):
+        async def handle_websocket():
+            ret = ''
 
-                report_ids = [i[1]['reportId'] for i in items]
-                requests_by_id.update(zip(report_ids, (i[0] for i in items)))
-                await asyncio.wait_for(ws.send(json.dumps(report_ids)), timeout=timeout)
-                dispatched.update(report_ids)
+            try:
+                # If we're re-connecting then re-send any in-flight request ids
 
-        async def read_from_websocket(ws: client):
-            outstanding_report_ids = [i for i in requests_by_id.keys() if i not in dispatched]
-            if outstanding_report_ids:
-                await asyncio.wait_for(ws.send(json.dumps(outstanding_report_ids)), timeout=timeout)
+                outstanding_request_ids = [i for i in pending_requests.keys() if i not in dispatched]
+                if outstanding_request_ids:
+                    _logger.info(f'Re-sending {len(outstanding_request_ids)} requests')
+                    await asyncio.wait_for(ws.send(json.dumps(outstanding_request_ids)), timeout=send_timeout)
 
-            while True:
-                raw_result = await asyncio.wait_for(ws.recv(), timeout=timeout)
-                result_id, status_result_str = raw_result.split(';', 1)
-                status, result_str = status_result_str[0], status_result_str[1:]
+                all_requests_dispatched = False
 
-                if status == 'E':
-                    result = RuntimeError(result_str)
-                else:
-                    try:
-                        result = msgpack.unpackb(base64.b64decode(result_str), raw=False) if cls.USE_MSGPACK else\
-                            json.loads(result_str)
-                    except Exception as ee:
-                        result = ee
+                while pending_requests or not all_requests_dispatched:
+                    # Continue while we have pending or un-dispatched requests
 
-                request = requests_by_id.pop(result_id)
-                results.put_nowait((request, result))
+                    request_listener = asyncio.ensure_future(cls.drain_queue_async(responses))\
+                        if not all_requests_dispatched else None
+                    result_listener = asyncio.ensure_future(ws.recv())
+                    listeners = tuple(filter(None, (request_listener, result_listener)))
 
-        requests_by_id = {}
+                    # Wait for either a request or result
+                    complete, pending = await asyncio.wait(listeners, return_when=asyncio.FIRST_COMPLETED)
+
+                    # Check results before sending more requests. Results can be lost otherwise
+
+                    if result_listener in complete:
+                        # New results have been received
+
+                        request_id, status_result_str = result_listener.result().split(';', 1)
+                        status, result_str = status_result_str[0], status_result_str[1:]
+
+                        if status == 'E':
+                            # An error
+                            result = RuntimeError(result_str)
+                        else:
+                            # Unpack the result
+
+                            try:
+                                result = msgpack.unpackb(base64.b64decode(result_str), raw=False)\
+                                    if cls.USE_MSGPACK else json.loads(result_str)
+                            except Exception as ee:
+                                result = ee
+
+                        # Enqueue the request and result for the listener to handle
+                        results.put_nowait((pending_requests.pop(request_id), result))
+                    else:
+                        result_listener.cancel()
+
+                    if request_listener:
+                        if request_listener in complete:
+                            # New requests have been posted ...
+
+                            all_requests_dispatched, items = request_listener.result()
+                            if items:
+                                # ... extract the request IDs ...
+                                request_ids = [i[1]['reportId'] for i in items]
+
+                                # ... update the pending requests ...
+                                pending_requests.update(zip(request_ids, (i[0] for i in items)))
+
+                                # ... add to our result subscription ...
+
+                                await asyncio.wait_for(ws.send(json.dumps(request_ids)), timeout=send_timeout)
+
+                                # ... note dispatched
+                                dispatched.update(request_ids)
+                        else:
+                            request_listener.cancel()
+            except Exception as e:
+                ret = str(e)
+
+            return ret
+
+        pending_requests = {}
         dispatched = set()
         error = ''
         attempts = 0
         max_attempts = 5
+        send_timeout = 30
 
         while attempts < max_attempts:
             if attempts > 0:
                 await asyncio.sleep(math.pow(2, attempts))
-                _logger.error('{} error, retrying (attempt {} of {})'.format(error, attempts + 1, max_attempts))
+                _logger.error(f'{error} error, retrying (attempt {attempts + 1} of {max_attempts})')
 
             try:
                 async with GsSession.current._connect_websocket('/risk/calculate/results/subscribe') as ws:
-                    await asyncio.gather(
-                        send_to_websocket(ws),
-                        read_from_websocket(ws)
-                    )
-                    attempts = max_attempts
-                    error = ''
+                    error = await handle_websocket()
+
+                attempts = max_attempts
             except ConnectionClosedError as cce:
                 error = 'Connection failed: ' + str(cce)
                 attempts += 1
@@ -131,10 +169,11 @@ class GsRiskApi(RiskApi):
                 attempts = max_attempts
 
         if error != '':
-            results.put_nowait([])
+            _logger.error(f'Fatal error: {error}')
+            cls.shutdown_queue_listener(results)
 
     @classmethod
-    def create_pretrade_execution_optimization(cls, request: APEXOptimizationRequest) -> str:
+    def create_pretrade_execution_optimization(cls, request: OptimizationRequest) -> str:
         try:
             response = GsSession.current._post(r'/risk/execution/pretrade', request)
             _logger.info('New optimization is created with id: {}'.format(response.get("optimizationId")))
