@@ -13,25 +13,32 @@ KIND, either express or implied.  See the License for the
 specific language governing permissions and limitations
 under the License.
 """
-from abc import ABCMeta
 import builtins
-from collections import namedtuple
 import copy
 import datetime as dt
-import dateutil
-from enum import EnumMeta
-from functools import wraps
-import inflection
-from inspect import signature, Parameter
 import itertools
 import keyword
 import logging
+from abc import ABCMeta, abstractmethod
+from collections import namedtuple
+from enum import EnumMeta
+from functools import wraps
+from inspect import signature, Parameter
 from typing import Optional, Union, get_type_hints
 
-from gs_quant.context_base import ContextBase, ContextMeta
+import inflection
+from dateutil.parser import isoparse
+import numpy as np
+
+from gs_quant.context_base import ContextBase, ContextMeta, do_not_serialise
 
 _logger = logging.getLogger(__name__)
 
+_valid_date_formats = ('%Y-%m-%d',  # '2020-07-28'
+                       '%d%b%y',    # '28Jul20'
+                       '%d-%b-%y',  # '28-Jul-20'
+                       '%d/%m/%Y')  # '28/07/2020
+
 
 def _normalise_arg(arg: str) -> str:
     if keyword.iskeyword(arg) or arg in dir(builtins):
@@ -67,9 +74,17 @@ def camel_case_translate(f):
 class RiskKey(namedtuple('RiskKey', ('provider', 'date', 'market', 'params', 'scenario', 'risk_measure'))):
 
     @property
-    def base(self):
+    def ex_date_and_market(self):
         return RiskKey(self.provider, None, None, self.params, self.scenario, self.risk_measure)
 
+    @property
+    def ex_measure(self):
+        return RiskKey(self.provider, self.date, self.market, self.params, self.scenario, None)
+
+    @property
+    def fields(self):
+        return self._fields
+
 
 class EnumBase:
 
@@ -77,17 +92,16 @@ class EnumBase:
     def _missing_(cls: EnumMeta, key):
         return next((m for m in cls.__members__.values() if m.value.lower() == key.lower()), None)
 
-    def __lt__(self, other):
+    def __lt__(self: EnumMeta, other):
         return self.value < other.value
 
 
 class Base(metaclass=ABCMeta):
-
     """The base class for all generated classes"""
 
     __properties = set()
 
-    def __init__(self):
+    def __init__(self, **kwargs):
         self.__calced_hash: Optional[int] = None
         self.__as_dict = {False: {}, True: {}}
 
@@ -133,15 +147,20 @@ class Base(metaclass=ABCMeta):
         if not self._hash_is_calced:
             calced_hash = hash(self.name)
             for prop in self.properties():
-                calced_hash ^= hash(super().__getattribute__(prop))
+                value = super().__getattribute__(prop)
+                if isinstance(value, dict):
+                    value = tuple(value.items())
+                elif isinstance(value, list):
+                    value = tuple(value)
+                calced_hash ^= hash(value)
 
             self.__calced_hash = calced_hash
 
         return self.__calced_hash
 
     def __eq__(self, other) -> bool:
-        return\
-            type(self) == type(other) and self.name == other.name and\
+        return \
+            type(self) == type(other) and (self.name is None or other.name is None or self.name == other.name) and \
             (self.__calced_hash is None or other.__calced_hash is None or self.__calced_hash == other.__calced_hash) \
             and all(super(Base, self).__getattribute__(p) == super(Base, other).__getattribute__(p)
                     for p in self.properties())
@@ -150,9 +169,12 @@ class Base(metaclass=ABCMeta):
         return not self.__eq__(other)
 
     def __lt__(self, other):
+        if not isinstance(other, type(self)):
+            return type(self).__name__ < type(other).__name__
+
         for prop in itertools.chain(('name',), sorted(self.properties())):
-            val = getattr(self, prop)
-            other_val = getattr(other, prop)
+            val = super(Base, self).__getattribute__(prop)
+            other_val = super(Base, other).__getattribute__(prop)
 
             if val != other_val:
                 if val is None:
@@ -204,7 +226,7 @@ class Base(metaclass=ABCMeta):
             values = (super(Base, self).__getattribute__(p) for p in raw_properties)
             self.__as_dict[as_camel_case] = dict((p, v) for p, v in zip(properties, values) if v is not None)
 
-        return self.__as_dict[as_camel_case]
+        return copy.copy(self.__as_dict[as_camel_case])
 
     @classmethod
     def prop_type(cls, prop: str, additional: Optional[list] = None) -> type:
@@ -225,7 +247,7 @@ class Base(metaclass=ABCMeta):
                             additional.extend([a for a in return_hints.__args__ if a != prop_type])
                         break
 
-            if prop_type is None:
+            if prop_type is None and additional is not None:
                 prop_type = return_hints.__args__[-1]
                 additional.extend(return_hints.__args__[:-1])
 
@@ -255,6 +277,9 @@ class Base(metaclass=ABCMeta):
             prop_value = values.get(prop, values.get(inflection.camelize(prop, uppercase_first_letter=False)))
 
             if prop_value is not None:
+                if isinstance(prop_value, np.generic):
+                    prop_value = prop_value.item()
+
                 additional_types = []
                 prop_type = self.prop_type(prop, additional=additional_types)
 
@@ -273,16 +298,32 @@ class Base(metaclass=ABCMeta):
                                 prop_value = re.sub(matcher.re, '.{}Z'.format(sub_seconds[:6]), prop_value)
 
                         try:
-                            setattr(self, prop, dateutil.parser.isoparse(prop_value))
+                            setattr(self, prop, isoparse(prop_value))
                         except ValueError:
                             if str in additional_types:
                                 setattr(self, prop, prop_value)
-                elif issubclass(prop_type, dt.date):
-                    try:
-                        setattr(self, prop, dateutil.parser.isoparse(prop_value).date())
-                    except ValueError:
-                        if str in additional_types:
-                            setattr(self, prop, prop_value)
+                elif issubclass(prop_type, dt.date) and type(prop_value) is not dt.date:
+                    date_value = None
+
+                    if isinstance(prop_value, float):
+                        # Assume it's an Excel date
+                        if prop_value > 59:
+                            prop_value -= 1  # Excel leap year bug, 1900 is not a leap year!
+                        date_value = dt.datetime(1899, 12, 31) + dt.timedelta(days=prop_value).date()
+                    elif isinstance(prop_value, str):
+                        for format in _valid_date_formats:
+                            try:
+                                date_value = dt.datetime.strptime(prop_value, format).date()
+                                break
+                            except ValueError:
+                                pass
+
+                    setattr(self, prop, date_value or prop_value)
+                elif issubclass(prop_type, float) and isinstance(prop_value, str):
+                    if prop_value.endswith('%'):
+                        setattr(self, prop, float(prop_value[:-1]) / 100)
+                    else:
+                        setattr(self, prop, float(prop_value))
                 elif issubclass(prop_type, EnumBase):
                     setattr(self, prop, get_enum_value(prop_type, prop_value))
                 elif issubclass(prop_type, Base):
@@ -304,7 +345,7 @@ class Base(metaclass=ABCMeta):
                     setattr(self, prop, prop_value)
 
     @classmethod
-    def _from_dict(cls, values: dict) -> 'Base':
+    def _from_dict(cls, values: dict):
         args = [k for k, v in signature(cls.__init__).parameters.items() if k not in ('kwargs', '_kwargs')
                 and v.default == Parameter.empty][1:]
         required = {}
@@ -366,24 +407,6 @@ class Base(metaclass=ABCMeta):
 
 class Priceable(Base, metaclass=ABCMeta):
 
-    PROVIDER = None
-
-    def provider(self):
-        """
-        The risk provider - defaults to GsRiskApi
-        """
-        if self.PROVIDER is None:
-            from gs_quant.api.gs.risk import GsRiskApi
-            type(self).PROVIDER = GsRiskApi
-
-        return self.PROVIDER
-
-    def get_quantity(self) -> float:
-        """
-        Quantity of the instrument
-        """
-        raise NotImplementedError
-
     def resolve(self, in_place: bool = True):
         """
         Resolve non-supplied properties of an instrument
@@ -450,11 +473,12 @@ class Priceable(Base, metaclass=ABCMeta):
         """
         raise NotImplementedError
 
-    def calc(self, risk_measure):
+    def calc(self, risk_measure, fn=None):
         """
         Calculate the value of the risk_measure
 
         :param risk_measure: the risk measure to compute, e.g. IRDelta (from gs_quant.risk)
+        :param fn: a function for post-processing results
         :return: a float or dataframe, depending on whether the value is scalar or structured, or a future thereof
         (depending on how PricingContext is being used)
 
@@ -502,17 +526,89 @@ class Scenario(Base, ContextBase, metaclass=__ScenarioMeta):
 
 
 class InstrumentBase(Base):
-    pass
+
+    def __init__(self, quantity: Optional[float] = 1):
+        super().__init__()
+        self.__instrument_quantity = quantity
+        self.__resolution_key: Optional[RiskKey] = None
+        self.__unresolved: Optional[InstrumentBase] = None
+
+    @property
+    @abstractmethod
+    @do_not_serialise
+    def provider(self):
+        ...
+
+    @property
+    @do_not_serialise
+    def instrument_quantity(self) -> float:
+        return self.__instrument_quantity
+
+    @property
+    @do_not_serialise
+    def resolution_key(self) -> RiskKey:
+        return self.__resolution_key
+
+    @property
+    @do_not_serialise
+    def unresolved(self):
+        return self.__unresolved
+
+    def _property_changed(self, prop: str):
+        if self.__resolution_key:
+            self.__resolution_key = None
+            self.unresolve()
+
+        super()._property_changed(prop)
+
+    def from_instance(self, instance):
+        self.__resolution_key = None
+        super().from_instance(instance)
+        self.__unresolved = instance.__unresolved
+        self.__resolution_key = instance.__resolution_key
+
+    def resolved(self, values: dict, resolution_key: RiskKey):
+        new_instrument = self.from_dict(values)
+        new_instrument.name = self.name
+        new_instrument.__unresolved = copy.copy(self)
+        new_instrument.__resolution_key = resolution_key
+        return new_instrument
+
+    def unresolve(self):
+        if self.__resolution_key and self.__unresolved:
+            self.from_instance(self.__unresolved)
+            self.__resolution_key = None
+            self.__unresolved = None
 
 
-class QuotableBuilder(Base):
-    pass
+class QuotableBuilder(Base, metaclass=ABCMeta):
+
+    def __init__(self, valuation_overrides: Optional[dict] = None):
+        super().__init__()
+        self.valuation_overrides = valuation_overrides
+
+    @property
+    def valuation_overrides(self) -> Optional[dict]:
+        return self.__valuation_overrides
+
+    @valuation_overrides.setter
+    def valuation_overrides(self, value: dict):
+        self.__valuation_overrides = value
 
 
 class Market(Base):
     pass
 
 
+class Sentinel:
+
+    def __init__(self, name: str):
+        self.__name = name
+
+    def __eq__(self, other):
+        return self.__name == other.__name
+
+
 def get_enum_value(enum_type: EnumMeta, value: Union[EnumBase, str]):
     if value in (None,):
         return None
