@@ -53,6 +53,9 @@ class Dataset:
         TR = 'TR'
         TR_FXSPOT = 'TR_FXSPOT'
 
+    class FRED(Vendor):
+        GDP = 'GDP'
+
     def __init__(self, dataset_id: Union[str, Vendor], provider: DataApi = None):
         """
 
@@ -181,6 +184,8 @@ class Dataset:
             if len(gb.groups) > 1:
                 raise MqValueError('Not a series for a single {}'.format(symbol_dimension))
 
+        if df.empty:
+            return pd.Series()
         return pd.Series(index=df.index, data=df.loc[:, field_value].values)
 
     def get_data_last(
@@ -211,6 +216,7 @@ class Dataset:
             start=start,
             end=as_of,
             fields=fields,
+            format='JSON',
             **kwargs
         )
         query.format = None  # "last" endpoint does not support MessagePack
@@ -223,6 +229,7 @@ class Dataset:
             limit: int = None,
             offset: int = None,
             fields: List[str] = None,
+            include_history: bool = False,
             **kwargs
     ) -> pd.DataFrame:
         """
@@ -231,6 +238,7 @@ class Dataset:
         :param limit: The maximum number of assets to return
         :param offset: The offset
         :param fields: The fields to return, e.g. assetId
+        :param include_history: Return column for historyStartDate
         :return: A Dataframe of the assets covered
 
         **Examples**
@@ -245,6 +253,7 @@ class Dataset:
             limit=limit,
             offset=offset,
             fields=fields,
+            include_history=include_history,
             **kwargs
         )
 
