@@ -34,7 +34,15 @@ def default(o):
         return o.to_json()
     elif isinstance(o, Base):
         properties = {re.sub('_$', '', k): v for k, v in o.as_dict(as_camel_case=True).items()}
-        return {'$type': o._type, 'properties': properties} if isinstance(o, QuotableBuilder) else properties
+
+        if isinstance(o, QuotableBuilder):
+            ret = {'$type': o._type, 'properties': properties}
+            if 'valuationOverrides' in properties:
+                ret['valuationOverrides'] = properties.pop('valuationOverrides')
+
+            return ret
+        else:
+            return properties
 
 
 class JSONEncoder(json.JSONEncoder):
