@@ -15,17 +15,26 @@ under the License.
 """
 
 import datetime as dt
+import json
 import threading
 from abc import ABCMeta, abstractmethod
 from enum import Enum
-from typing import Optional, Tuple, Union
+from typing import Optional, Tuple, Union, Dict
 
 import cachetools
+import pandas as pd
 import pytz
 
-from gs_quant.api.gs.assets import GsAssetApi, GsAsset, AssetClass, AssetType as GsAssetType, PositionSet
+from gs_quant.api.gs.assets import GsAssetApi, GsAsset, AssetClass, AssetParameters, \
+    AssetType as GsAssetType, PositionSet, Currency
 from gs_quant.base import get_enum_value
 from gs_quant.common import PositionType
+from gs_quant.data import DataMeasure, DataFrequency
+from gs_quant.data.coordinate import DataDimensions
+from gs_quant.data.coordinate import DateOrDatetime
+from gs_quant.entities.entity import Entity, EntityIdentifier, EntityType
+from gs_quant.errors import MqValueError
+from gs_quant.json_encoder import JSONEncoder
 from gs_quant.markets import PricingContext
 
 
@@ -90,8 +99,26 @@ class AssetType(Enum):
     #: Commodity Reference Price
     COMMODITY_REFERENCE_PRICE = "Commodity Reference Price"
 
+    #: Commodity Power Node
+    COMMODITY_POWER_NODE = "Commodity Power Node"
 
-class AssetIdentifier(Enum):
+    #: Commodity Power Aggregated Nodes
+    COMMODITY_POWER_AGGREGATED_NODES = "Commodity Power Aggregated Nodes"
+
+    #: Bond
+    BOND = "Bond"
+
+    #: Future Market
+    FUTURE_MARKET = "Future Market"
+
+    #: Future Contract
+    FUTURE_CONTRACT = "Future Contract"
+
+    #: Commodity
+    COMMODITY = "Commodity"
+
+
+class AssetIdentifier(EntityIdentifier):
     """Asset type enumeration
 
     Enumeration of different security identifiers
@@ -108,12 +135,33 @@ class AssetIdentifier(Enum):
     TICKER = "TICKER"  #: Exchange ticker (GS)
 
 
-class Asset(metaclass=ABCMeta):
-    def __init__(self, id_: str, asset_class: AssetClass, name: str, exchange: Optional[str] = None):
+class ReturnType(Enum):
+    """Index return type
+
+    Represents different index return types or funding models
+
+    """
+
+    EXCESS_RETURN = "Excess Return"  # Returns are excess of funding rate in denominated currency
+    TOTAL_RETURN = "Total Return"  # Returns are inclusive of funding rate in denominated currency
+
+
+class Asset(Entity, metaclass=ABCMeta):
+    def __init__(self,
+                 id_: str,
+                 asset_class: AssetClass,
+                 name: str,
+                 exchange: Optional[str] = None,
+                 currency: Optional[str] = None,
+                 parameters: AssetParameters = None,
+                 entity: Optional[Dict] = None):
+        super().__init__(id_, EntityType.ASSET, entity=entity)
         self.__id = id_
         self.asset_class = asset_class
         self.name = name
         self.exchange = exchange
+        self.currency = currency
+        self.parameters = parameters
 
     def get_marquee_id(self):
         return self.__id
@@ -151,7 +199,6 @@ class Asset(metaclass=ABCMeta):
 
         :class:`AssetIdentifier`
         :func:`get_asset`
-
         """
         if not as_of:
             as_of = PricingContext.current.pricing_date
@@ -213,13 +260,106 @@ class Asset(metaclass=ABCMeta):
         :func:`get_asset_identifiers`
 
         """
+        if id_type == AssetIdentifier.MARQUEE_ID:
+            return self.__id
+
         ids = self.get_identifiers(as_of=as_of)
         return ids.get(id_type.value)
 
+    def get_data_series(self,
+                        measure: DataMeasure,
+                        dimensions: Optional[DataDimensions] = None,
+                        frequency: Optional[DataFrequency] = None,
+                        start: Optional[DateOrDatetime] = None,
+                        end: Optional[DateOrDatetime] = None) -> pd.Series:
+        """
+        Get asset series
+
+        :param measure: measure to get as series
+        :param dimensions: dimensions to query (e.g. tenor)
+        :param frequency: data frequency to query
+        :param start: start of the series
+        :param end: end of the series
+        :return: timeseries of given measure
+
+        **Usage**
+
+        Get a given timeseries for the asset
+
+        **Examples**
+
+        Get close price series:
+
+        >>> from gs_quant.markets.securities import SecurityMaster
+        >>> from gs_quant.data import DataMeasure
+        >>>
+        >>> gs = SecurityMaster.get_asset("GS", AssetIdentifier.TICKER)
+        >>> gs.get_data_series(DataMeasure.CLOSE_PRICE)
+
+        **See also**
+
+        :class:`DataMeasure`
+
+        """
+
+        coordinate = self.get_data_coordinate(measure, dimensions, frequency)
+        if coordinate is None:
+            raise MqValueError(f"No data co-ordinate found for these parameters: {measure, dimensions, frequency}")
+        return coordinate.get_series(start=start, end=end)
+
+    def get_close_prices(self,
+                         start: Optional[dt.date] = None,
+                         end: Optional[dt.date] = None) -> pd.Series:
+        """
+        Get close price series
+
+        :return: timeseries of close prices
+
+        **Usage**
+
+        Get close prices for an asset
+
+        **Examples**
+
+        Get close price series:
+
+        >>> from gs_quant.markets.securities import SecurityMaster
+        >>>
+        >>> gs = SecurityMaster.get_asset("GS", AssetIdentifier.TICKER)
+        >>> gs.get_close_prices()
+
+        **See also**
+
+        :class:`DataMeasure`
+        :func:`get_data_series`
+
+        """
+
+        return self.get_data_series(DataMeasure.CLOSE_PRICE, None, DataFrequency.DAILY, start, end)
+
     @abstractmethod
     def get_type(self) -> AssetType:
         """Overridden by sub-classes to return security type"""
 
+    @property
+    def data_dimension(self) -> str:
+        return 'assetId'
+
+    @classmethod
+    def entity_type(cls) -> EntityType:
+        return EntityType.ASSET
+
+    @classmethod
+    def get(cls,
+            id_value: str,
+            id_type: AssetIdentifier,
+            as_of: Union[dt.date, dt.datetime] = None,
+            exchange_code: ExchangeCode = None,
+            asset_type: AssetType = None,
+            sort_by_rank: bool = False) -> Optional['Asset']:
+        asset = SecurityMaster.get_asset(id_value, id_type, as_of, exchange_code, asset_type, sort_by_rank)
+        return asset
+
 
 class Stock(Asset):
     """Base Security Type
@@ -229,12 +369,20 @@ class Stock(Asset):
 
     """
 
-    def __init__(self, id_: str, name: str, exchange=None):
-        Asset.__init__(self, id_, AssetClass.Equity, name, exchange)
+    def __init__(self,
+                 id_: str,
+                 name: str,
+                 exchange: Optional[str] = None,
+                 currency: Optional[Currency] = None,
+                 entity: Optional[Dict] = None):
+        Asset.__init__(self, id_, AssetClass.Equity, name, exchange, currency, entity=entity)
 
     def get_type(self) -> AssetType:
         return AssetType.STOCK
 
+    def get_currency(self) -> Optional[Currency]:
+        return self.currency
+
 
 class Cross(Asset):
     """Base Security Type
@@ -244,8 +392,11 @@ class Cross(Asset):
 
     """
 
-    def __init__(self, id_: str, name: str):
-        Asset.__init__(self, id_, AssetClass.FX, name)
+    def __init__(self,
+                 id_: str,
+                 name: str,
+                 entity: Optional[Dict] = None):
+        Asset.__init__(self, id_, AssetClass.FX, name, entity=entity)
 
     def get_type(self) -> AssetType:
         return AssetType.CROSS
@@ -259,15 +410,23 @@ class Future(Asset):
 
     """
 
-    def __init__(self, id_: str, asset_class: Union[AssetClass, str], name: str):
+    def __init__(self,
+                 id_: str,
+                 asset_class: Union[AssetClass, str],
+                 name: str,
+                 currency: Optional[Currency] = None,
+                 entity: Optional[Dict] = None):
         if isinstance(asset_class, str):
             asset_class = get_enum_value(AssetClass, asset_class)
 
-        Asset.__init__(self, id_, asset_class, name)
+        Asset.__init__(self, id_, asset_class, name, currency=currency, entity=entity)
 
     def get_type(self) -> AssetType:
         return AssetType.FUTURE
 
+    def get_currency(self) -> Optional[Currency]:
+        return self.currency
+
 
 class Currency(Asset):
     """Base Security Type
@@ -277,8 +436,11 @@ class Currency(Asset):
 
     """
 
-    def __init__(self, id_: str, name: str):
-        Asset.__init__(self, id_, AssetClass.Cash, name)
+    def __init__(self,
+                 id_: str,
+                 name: str,
+                 entity: Optional[Dict] = None):
+        Asset.__init__(self, id_, AssetClass.Cash, name, entity=entity)
 
     def get_type(self) -> AssetType:
         return AssetType.CURRENCY
@@ -292,8 +454,11 @@ class Rate(Asset):
 
     """
 
-    def __init__(self, id_: str, name: str):
-        Asset.__init__(self, id_, AssetClass.Rates, name)
+    def __init__(self,
+                 id_: str,
+                 name: str,
+                 entity: Optional[Dict] = None):
+        Asset.__init__(self, id_, AssetClass.Rates, name, entity=entity)
 
     def get_type(self) -> AssetType:
         return AssetType.RATE
@@ -307,8 +472,11 @@ class Cash(Asset):
 
     """
 
-    def __init__(self, id_: str, name: str):
-        Asset.__init__(self, id_, AssetClass.Cash, name)
+    def __init__(self,
+                 id_: str,
+                 name: str,
+                 entity: Optional[Dict] = None):
+        Asset.__init__(self, id_, AssetClass.Cash, name, entity=entity)
 
     def get_type(self) -> AssetType:
         return AssetType.CASH
@@ -322,8 +490,11 @@ class WeatherIndex(Asset):
 
     """
 
-    def __init__(self, id_: str, name: str):
-        Asset.__init__(self, id_, AssetClass.Commod, name)
+    def __init__(self,
+                 id_: str,
+                 name: str,
+                 entity: Optional[Dict] = None):
+        Asset.__init__(self, id_, AssetClass.Commod, name, entity=entity)
 
     def get_type(self) -> AssetType:
         return AssetType.WEATHER_INDEX
@@ -337,13 +508,124 @@ class CommodityReferencePrice(Asset):
 
     """
 
-    def __init__(self, id_: str, name: str):
-        Asset.__init__(self, id_, AssetClass.Commod, name)
+    def __init__(self,
+                 id_: str,
+                 name: str,
+                 entity: Optional[Dict] = None):
+        Asset.__init__(self, id_, AssetClass.Commod, name, entity=entity)
 
     def get_type(self) -> AssetType:
         return AssetType.COMMODITY_REFERENCE_PRICE
 
 
+class CommodityPowerNode(Asset):
+    """Commodity Power Node
+
+    Represents a distinct location in commodity power markets
+
+    """
+
+    def __init__(self,
+                 id_: str,
+                 name: str,
+                 entity: Optional[Dict] = None):
+        Asset.__init__(self, id_, AssetClass.Commod, name, entity=entity)
+
+    def get_type(self) -> AssetType:
+        return AssetType.COMMODITY_POWER_NODE
+
+
+class CommodityPowerAggregatedNodes(Asset):
+    """Commodity Power Aggregated Nodes
+
+    Represents a group of locations in commodity power markets
+
+    """
+
+    def __init__(self,
+                 id_: str,
+                 name: str,
+                 entity: Optional[Dict] = None):
+        Asset.__init__(self, id_, AssetClass.Commod, name, entity=entity)
+
+    def get_type(self) -> AssetType:
+        return AssetType.COMMODITY_POWER_AGGREGATED_NODES
+
+
+class Commodity(Asset):
+    """Commodity
+
+    Represents a commodity.
+
+    """
+
+    def __init__(self,
+                 id_: str,
+                 name: str,
+                 entity: Optional[Dict] = None):
+        Asset.__init__(self, id_, AssetClass.Commod, name, entity=entity)
+
+    def get_type(self) -> AssetType:
+        return AssetType.COMMODITY
+
+
+class Bond(Asset):
+    """Bond
+
+    Represents a bond.
+
+    """
+
+    def __init__(self,
+                 id_: str,
+                 name: str,
+                 entity: Optional[Dict] = None):
+        Asset.__init__(self, id_, AssetClass.Credit, name, entity=entity)
+
+    def get_type(self) -> AssetType:
+        return AssetType.BOND
+
+
+class FutureMarket(Asset):
+    """Future Market
+
+    Represents a future market
+
+    """
+
+    def __init__(self,
+                 id_: str,
+                 asset_class: Union[AssetClass, str],
+                 name: str,
+                 entity: Optional[Dict] = None):
+        if isinstance(asset_class, str):
+            asset_class = get_enum_value(AssetClass, asset_class)
+        Asset.__init__(self, id_, asset_class, name, entity=entity)
+
+    def get_type(self) -> AssetType:
+        return AssetType.FUTURE_MARKET
+
+
+class FutureContract(Asset):
+    """Future Contract
+
+    Represents a future contract
+
+    """
+
+    def __init__(self,
+                 id_: str,
+                 asset_class: Union[AssetClass, str],
+                 name: Union[AssetClass, str],
+                 entity: Optional[Dict] = None):
+        if isinstance(asset_class, str):
+            asset_class = get_enum_value(AssetClass, asset_class)
+        Asset.__init__(self, id_, asset_class, name, entity=entity)
+
+    def get_type(self) -> AssetType:
+        return AssetType.FUTURE_CONTRACT
+
+
 class Swap(Asset):
     """Swap Instrument Type
 
@@ -351,11 +633,15 @@ class Swap(Asset):
 
     """
 
-    def __init__(self, id_: str, asset_class: Union[AssetClass, str], name: str):
+    def __init__(self,
+                 id_: str,
+                 asset_class: Union[AssetClass, str],
+                 name: str,
+                 entity: Optional[Dict] = None):
         if isinstance(asset_class, str):
             asset_class = get_enum_value(AssetClass, asset_class)
 
-        Asset.__init__(self, id_, asset_class, name)
+        Asset.__init__(self, id_, asset_class, name, entity=entity)
 
     def get_type(self) -> AssetType:
         return AssetType.SWAP
@@ -368,11 +654,15 @@ class Option(Asset):
 
     """
 
-    def __init__(self, id_: str, asset_class: Union[AssetClass, str], name: str):
+    def __init__(self,
+                 id_: str,
+                 asset_class: Union[AssetClass, str],
+                 name: str,
+                 entity: Optional[Dict] = None):
         if isinstance(asset_class, str):
             asset_class = get_enum_value(AssetClass, asset_class)
 
-        Asset.__init__(self, id_, asset_class, name)
+        Asset.__init__(self, id_, asset_class, name, entity=entity)
 
     def get_type(self) -> AssetType:
         return AssetType.OPTION
@@ -435,13 +725,28 @@ class Index(Asset, IndexConstituentProvider):
     Index which tracks an evolving portfolio of securities, and can be traded through cash or derivatives markets
     """
 
-    def __init__(self, id_: str, asset_class: AssetClass, name: str, exchange=None):
-        Asset.__init__(self, id_, asset_class, name, exchange)
+    def __init__(self,
+                 id_: str,
+                 asset_class: AssetClass,
+                 name: str,
+                 exchange: Optional[str] = None,
+                 currency: Optional[Currency] = None,
+                 entity: Optional[Dict] = None):
+        Asset.__init__(self, id_, asset_class, name, exchange, currency, entity=entity)
         IndexConstituentProvider.__init__(self, id_)
 
     def get_type(self) -> AssetType:
         return AssetType.INDEX
 
+    def get_currency(self) -> Optional[Currency]:
+        return self.currency
+
+    def get_return_type(self) -> ReturnType:
+        if self.parameters is None or self.parameters.index_return_type is None:
+            return ReturnType.TOTAL_RETURN
+
+        return ReturnType(self.parameters.index_return_type)
+
 
 class ETF(Asset, IndexConstituentProvider):
     """ETF Asset
@@ -449,13 +754,22 @@ class ETF(Asset, IndexConstituentProvider):
     ETF which tracks an evolving portfolio of securities, and can be traded on exchange
     """
 
-    def __init__(self, id_: str, asset_class: AssetClass, name: str, exchange=None):
-        Asset.__init__(self, id_, asset_class, name, exchange)
+    def __init__(self,
+                 id_: str,
+                 asset_class: AssetClass,
+                 name: str,
+                 exchange: Optional[str] = None,
+                 currency: Optional[Currency] = None,
+                 entity: Optional[Dict] = None):
+        Asset.__init__(self, id_, asset_class, name, exchange, currency, entity=entity)
         IndexConstituentProvider.__init__(self, id_)
 
     def get_type(self) -> AssetType:
         return AssetType.ETF
 
+    def get_currency(self) -> Optional[Currency]:
+        return self.currency
+
 
 class Basket(Asset, IndexConstituentProvider):
     """Basket Asset
@@ -463,13 +777,21 @@ class Basket(Asset, IndexConstituentProvider):
     Basket which tracks an evolving portfolio of securities, and can be traded through cash or derivatives markets
     """
 
-    def __init__(self, id_: str, asset_class: AssetClass, name: str):
-        Asset.__init__(self, id_, asset_class, name)
+    def __init__(self,
+                 id_: str,
+                 asset_class: AssetClass,
+                 name: str,
+                 currency: Optional[Currency] = None,
+                 entity: Optional[Dict] = None):
+        Asset.__init__(self, id_, asset_class, name, currency=currency, entity=entity)
         IndexConstituentProvider.__init__(self, id_)
 
     def get_type(self) -> AssetType:
         return AssetType.BASKET
 
+    def get_currency(self) -> Optional[Currency]:
+        return self.currency
+
 
 class SecurityMaster:
     """Security Master
@@ -489,51 +811,72 @@ class SecurityMaster:
     @classmethod
     def __gs_asset_to_asset(cls, gs_asset: GsAsset) -> Asset:
         asset_type = gs_asset.type.value
+        asset_entity: Dict = json.loads(json.dumps(gs_asset.as_dict(), cls=JSONEncoder))
 
         if asset_type in (GsAssetType.Single_Stock.value,):
-            return Stock(gs_asset.id, gs_asset.name, gs_asset.exchange)
+            return Stock(gs_asset.id, gs_asset.name, gs_asset.exchange, gs_asset.currency, entity=asset_entity)
 
         if asset_type in (GsAssetType.ETF.value,):
-            return ETF(gs_asset.id, gs_asset.assetClass, gs_asset.name, gs_asset.exchange)
+            return ETF(gs_asset.id, gs_asset.assetClass, gs_asset.name, gs_asset.exchange, gs_asset.currency,
+                       entity=asset_entity)
 
         if asset_type in (
                 GsAssetType.Index.value,
                 GsAssetType.Risk_Premia.value,
                 GsAssetType.Access.value,
                 GsAssetType.Multi_Asset_Allocation.value):
-            return Index(gs_asset.id, gs_asset.assetClass, gs_asset.name, gs_asset.exchange)
+            return Index(gs_asset.id, gs_asset.assetClass, gs_asset.name, gs_asset.exchange, gs_asset.currency,
+                         entity=asset_entity)
 
         if asset_type in (
                 GsAssetType.Custom_Basket.value,
                 GsAssetType.Research_Basket.value):
-            return Basket(gs_asset.id, gs_asset.assetClass, gs_asset.name)
+            return Basket(gs_asset.id, gs_asset.assetClass, gs_asset.name, gs_asset.currency, entity=asset_entity)
 
         if asset_type in (GsAssetType.Future.value,):
-            return Future(gs_asset.id, gs_asset.assetClass, gs_asset.name)
+            return Future(gs_asset.id, gs_asset.assetClass, gs_asset.name, gs_asset.currency, entity=asset_entity)
 
         if asset_type in (GsAssetType.Cross.value,):
-            return Cross(gs_asset.id, gs_asset.name)
+            return Cross(gs_asset.id, gs_asset.name, entity=asset_entity)
 
         if asset_type in (GsAssetType.Currency.value,):
-            return Currency(gs_asset.id, gs_asset.name)
+            return Currency(gs_asset.id, gs_asset.name, entity=asset_entity)
 
         if asset_type in (GsAssetType.Rate.value,):
-            return Rate(gs_asset.id, gs_asset.name)
+            return Rate(gs_asset.id, gs_asset.name, entity=asset_entity)
 
         if asset_type in (GsAssetType.Cash.value,):
-            return Cash(gs_asset.id, gs_asset.name)
+            return Cash(gs_asset.id, gs_asset.name, entity=asset_entity)
 
         if asset_type in (GsAssetType.WeatherIndex.value,):
-            return WeatherIndex(gs_asset.id, gs_asset.name)
+            return WeatherIndex(gs_asset.id, gs_asset.name, entity=asset_entity)
 
         if asset_type in (GsAssetType.Swap.value,):
-            return Swap(gs_asset.id, gs_asset.assetClass, gs_asset.name)
+            return Swap(gs_asset.id, gs_asset.assetClass, gs_asset.name, entity=asset_entity)
 
         if asset_type in (GsAssetType.Option.value,):
-            return Option(gs_asset.id, gs_asset.assetClass, gs_asset.name)
+            return Option(gs_asset.id, gs_asset.assetClass, gs_asset.name, entity=asset_entity)
 
         if asset_type in (GsAssetType.CommodityReferencePrice.value,):
-            return CommodityReferencePrice(gs_asset.id, gs_asset.name)
+            return CommodityReferencePrice(gs_asset.id, gs_asset.name, entity=asset_entity)
+
+        if asset_type in (GsAssetType.CommodityPowerNode.value,):
+            return CommodityPowerNode(gs_asset.id, gs_asset.name, entity=asset_entity)
+
+        if asset_type in (GsAssetType.CommodityPowerAggregatedNodes.value,):
+            return CommodityPowerAggregatedNodes(gs_asset.id, gs_asset.name, entity=asset_entity)
+
+        if asset_type in (GsAssetType.Bond.value,):
+            return Bond(gs_asset.id, gs_asset.name, entity=asset_entity)
+
+        if asset_type in (GsAssetType.Commodity.value,):
+            return Commodity(gs_asset.id, gs_asset.name, entity=asset_entity)
+
+        if asset_type in (GsAssetType.FutureMarket.value,):
+            return FutureMarket(gs_asset.id, gs_asset.name, entity=asset_entity)
+
+        if asset_type in (GsAssetType.FutureContract.value,):
+            return FutureContract(gs_asset.id, gs_asset.name, entity=asset_entity)
 
         raise TypeError(f'unsupported asset type {asset_type}')
 
