@@ -18,7 +18,7 @@ from concurrent.futures import Future
 from copy import copy
 import itertools
 import datetime as dt
-from typing import Iterable, Optional, Tuple, Union
+from typing import Iterable, Optional, Tuple, Union, Any, Callable
 
 import pandas as pd
 
@@ -34,37 +34,13 @@ __column_sort_fns = {
 __risk_columns = ('date', 'time', 'mkt_type', 'mkt_asset', 'mkt_class', 'mkt_point')
 
 
-class RiskResult:
-
-    def __init__(self, result, risk_measures: Iterable[RiskMeasure]):
-        self.__risk_measures = tuple(risk_measures)
-        self.__result = result
-
-    def done(self) -> bool:
-        return self.__result.done()
-
-    def result(self):
-        return self.__result.result()
-
-    def add_done_callback(self, cb):
-        self.__result.add_done_callback(cb)
-
-    @property
-    def risk_measures(self) -> Tuple[RiskMeasure]:
-        return self.__risk_measures
-
-    @property
-    def _result(self):
-        return self.__result
-
-
 class ResultInfo(metaclass=ABCMeta):
 
     def __init__(
-        self,
-        risk_key: RiskKey,
-        unit: Optional[dict] = None,
-        error: Optional[Union[str, dict]] = None
+            self,
+            risk_key: RiskKey,
+            unit: Optional[dict] = None,
+            error: Optional[Union[str, dict]] = None
     ):
         self.__risk_key = risk_key
         self.__unit = unit
@@ -99,9 +75,7 @@ class ResultInfo(metaclass=ABCMeta):
 
         for component in components:
             date = component.risk_key.date
-            risk_key = component.risk_key.base if risk_key is None else risk_key
-            if risk_key != component.risk_key.base:
-                raise RuntimeError('Cannot compose heterogeneous results')
+            risk_key = component.risk_key.ex_date_and_market if risk_key is None else risk_key
 
             if isinstance(component, (ErrorValue, Exception)):
                 errors[date] = component
@@ -145,7 +119,7 @@ class ScalarWithInfo(ResultInfo, metaclass=ABCMeta):
     @staticmethod
     def compose(components: Iterable):
         dates, values, errors, risk_key, unit = ResultInfo.composition_info(components)
-        return SeriesWithInfo(pd.Series(index=pd.DatetimeIndex(dates), data=values),
+        return SeriesWithInfo(pd.Series(index=pd.DatetimeIndex(dates).date, data=values),
                               risk_key=risk_key,
                               unit=unit,
                               error=errors)
@@ -165,8 +139,18 @@ class FloatWithInfo(ScalarWithInfo, float):
         return float(self)
 
     def __repr__(self):
+
         return self.error if self.error else float.__repr__(self)
 
+    def __add__(self, other):
+        if isinstance(other, FloatWithInfo):
+            if self.unit == other.unit:
+                return FloatWithInfo(combine_risk_key(self.risk_key, other.risk_key), self.raw_value + other.raw_value,
+                                     self.unit)
+            else:
+                raise ValueError('FloatWithInfo unit mismatch')
+        return super(FloatWithInfo, self).__add__(other)
+
 
 class StringWithInfo(ScalarWithInfo, str):
 
@@ -186,8 +170,7 @@ class StringWithInfo(ScalarWithInfo, str):
 
 
 class SeriesWithInfo(pd.Series, ResultInfo):
-
-    _internal_names = pd.DataFrame._internal_names +\
+    _internal_names = pd.DataFrame._internal_names + \
                       ['_ResultInfo__' + i for i in dir(ResultInfo) if isinstance(getattr(ResultInfo, i), property)]
     _internal_names_set = set(_internal_names)
 
@@ -203,7 +186,9 @@ class SeriesWithInfo(pd.Series, ResultInfo):
         ResultInfo.__init__(self, risk_key, unit=unit, error=error)
 
     def __repr__(self):
-        return self.error if self.error else pd.Series.__repr__(self)
+        if self.error:
+            return pd.Series.__repr__(self) + "\nErrors: " + str(self.error)
+        return pd.Series.__repr__(self)
 
     @property
     def _constructor(self):
@@ -219,7 +204,6 @@ class SeriesWithInfo(pd.Series, ResultInfo):
 
 
 class DataFrameWithInfo(pd.DataFrame, ResultInfo):
-
     _internal_names = pd.DataFrame._internal_names + \
                       ['_ResultInfo__' + i for i in dir(ResultInfo) if isinstance(getattr(ResultInfo, i), property)]
     _internal_names_set = set(_internal_names)
@@ -236,7 +220,9 @@ class DataFrameWithInfo(pd.DataFrame, ResultInfo):
         ResultInfo.__init__(self, risk_key, unit=unit, error=error)
 
     def __repr__(self):
-        return self.error if self.error else pd.DataFrame.__repr__(self)
+        if self.error:
+            return pd.DataFrame.__repr__(self) + "\nErrors: " + str(self.errors)
+        return pd.DataFrame.__repr__(self)
 
     @property
     def _constructor(self):
@@ -258,7 +244,7 @@ class DataFrameWithInfo(pd.DataFrame, ResultInfo):
         return DataFrameWithInfo(df, risk_key=risk_key, unit=unit, error=errors)
 
 
-def aggregate_risk(results: Iterable[Union[DataFrameWithInfo, Future]], threshold: Optional[float] = None)\
+def aggregate_risk(results: Iterable[Union[DataFrameWithInfo, Future]], threshold: Optional[float] = None) \
         -> pd.DataFrame:
     """
     Combine the results of multiple InstrumentBase.calc() calls, into a single result
@@ -297,7 +283,7 @@ def aggregate_risk(results: Iterable[Union[DataFrameWithInfo, Future]], threshol
     return sort_risk(result)
 
 
-ResultType = Union[dict, tuple, DataFrameWithInfo, FloatWithInfo, SeriesWithInfo]
+ResultType = Union[None, dict, tuple, DataFrameWithInfo, FloatWithInfo, SeriesWithInfo]
 
 
 def aggregate_results(results: Iterable[ResultType]) -> ResultType:
@@ -305,6 +291,9 @@ def aggregate_results(results: Iterable[ResultType]) -> ResultType:
     risk_key = None
     results = tuple(results)
 
+    if not len(results):
+        return None
+
     for result in results:
         if isinstance(result, Exception):
             raise Exception
@@ -361,8 +350,8 @@ def subtract_risk(left: DataFrameWithInfo, right: DataFrameWithInfo) -> pd.DataF
     >>>
     >>> delta_diff = subtract_risk(delta_today, delta_yday_f.result())
     """
-    assert(left.columns.names == right.columns.names)
-    assert('value' in left.columns.names)
+    assert (left.columns.names == right.columns.names)
+    assert ('value' in left.columns.names)
 
     right_negated = copy(right)
     right_negated.value *= -1
@@ -394,3 +383,18 @@ def sort_risk(df: pd.DataFrame, by: Tuple[str, ...] = __risk_columns) -> pd.Data
         result = result.set_index('date')
 
     return result
+
+
+def combine_risk_key(key_1: RiskKey, key_2: RiskKey) -> RiskKey:
+    """
+    Combine two risk keys (key_1, key_2) into a new RiskKey
+
+    :type key_1: RiskKey
+    :type key_2: RiskKey
+    """
+
+    def get_field_value(field_name: str): getattr(key_1, field_name) \
+        if getattr(key_1, field_name) == getattr(key_2, field_name) else None
+
+    return RiskKey(get_field_value("provider"), get_field_value("date"), get_field_value("market"),
+                   get_field_value("params"), get_field_value("scenario"), get_field_value("risk_measure"))
