@@ -23,6 +23,9 @@ from gs_quant.target.risk import RiskMeasure as __RiskMeasure, RiskMeasureType,
 
 class RiskMeasure(__RiskMeasure):
 
+    def __repr__(self):
+        return self.name or self.measure_type.name
+
     @property
     @do_not_serialise
     def pricing_context(self):
@@ -58,6 +61,29 @@ class PnlExplain(__RelativeRiskMeasure):
         super().__init__(to_market, measure_type=RiskMeasureType.PnlExplain, name=RiskMeasureType.PnlExplain.value)
 
 
+class PnlExplainClose(PnlExplain):
+
+    def __init__(self):
+        from gs_quant.markets import CloseMarket
+        super().__init__(CloseMarket())
+
+
+class PnlExplainLive(PnlExplain):
+
+    def __init__(self):
+        from gs_quant.markets import LiveMarket
+        super().__init__(LiveMarket())
+
+
+class PnlPredictLive(__RelativeRiskMeasure):
+
+    """ Pnl Predicted """
+
+    def __init__(self):
+        from gs_quant.markets import LiveMarket
+        super().__init__(LiveMarket(), measure_type=RiskMeasureType.PnlPredict, name=RiskMeasureType.PnlPredict.value)
+
+
 def __risk_measure_with_doc_string(name: str,
                                    doc: str,
                                    measure_type: RiskMeasureType,
@@ -166,6 +192,16 @@ IRDeltaParallelLocalCcy = __risk_measure_with_doc_string(
     'Interest Rate Parallel Delta (Local Ccy)',
     RiskMeasureType.ParallelDeltaLocalCcy,
     asset_class=AssetClass.Rates)
+IRDiscountDeltaParallel = __risk_measure_with_doc_string(
+    'IRDiscountDeltaParallel',
+    'Parallel Discount Delta',
+    RiskMeasureType.ParallelDiscountDelta,
+    asset_class=AssetClass.Rates)
+IRDiscountDeltaParallelLocalCcy = __risk_measure_with_doc_string(
+    'IRDiscountDeltaParallelLocalCcy',
+    'Parallel Discount Delta (Local Ccy)',
+    RiskMeasureType.ParallelDiscountDeltaLocalCcy,
+    asset_class=AssetClass.Rates)
 IRXccyDelta = __risk_measure_with_doc_string(
     'IRXccyDelta',
     'Cross-ccy Delta',
@@ -265,3 +301,8 @@ MarketDataAssets = __risk_measure_with_doc_string(
     'MarketDataAssets',
     RiskMeasureType.Market_Data_Assets
 )
+MarketData = __risk_measure_with_doc_string(
+    'Market Data',
+    'Market Data map of coordinates and values',
+    RiskMeasureType.Market_Data
+)
