@@ -124,10 +124,12 @@ class BacktestPerformanceDecomposition(Base):
     def __init__(
         self,
         name: str = None,
+        performance: Tuple[FieldValueMap, ...] = None,
         stats: PerformanceStats = None
     ):        
         super().__init__()
         self.name = name
+        self.performance = performance
         self.stats = stats
 
     @property
@@ -140,6 +142,16 @@ class BacktestPerformanceDecomposition(Base):
         self._property_changed('name')
         self.__name = value        
 
+    @property
+    def performance(self) -> Tuple[FieldValueMap, ...]:
+        """Backtest performance curve."""
+        return self.__performance
+
+    @performance.setter
+    def performance(self, value: Tuple[FieldValueMap, ...]):
+        self._property_changed('performance')
+        self.__performance = value        
+
     @property
     def stats(self) -> PerformanceStats:
         """Performance statistics."""
@@ -220,10 +232,12 @@ class BacktestRisk(Base):
     @camel_case_translate
     def __init__(
         self,
-        name: str = None
+        name: str = None,
+        timeseries: Tuple[FieldValueMap, ...] = None
     ):        
         super().__init__()
         self.name = name
+        self.timeseries = timeseries
 
     @property
     def name(self) -> str:
@@ -235,6 +249,16 @@ class BacktestRisk(Base):
         self._property_changed('name')
         self.__name = value        
 
+    @property
+    def timeseries(self) -> Tuple[FieldValueMap, ...]:
+        """Backtest portfolio risk curve."""
+        return self.__timeseries
+
+    @timeseries.setter
+    def timeseries(self, value: Tuple[FieldValueMap, ...]):
+        self._property_changed('timeseries')
+        self.__timeseries = value        
+
 
 class BacktestRiskPosition(Base):
         
@@ -389,11 +413,13 @@ class ComparisonBacktestResult(Base):
     def __init__(
         self,
         stats: PerformanceStats = None,
+        performance: Tuple[FieldValueMap, ...] = None,
         id_: str = None,
         name: str = None
     ):        
         super().__init__()
         self.stats = stats
+        self.performance = performance
         self.__id = id_
         self.name = name
 
@@ -407,6 +433,16 @@ class ComparisonBacktestResult(Base):
         self._property_changed('stats')
         self.__stats = value        
 
+    @property
+    def performance(self) -> Tuple[FieldValueMap, ...]:
+        """Performance for the comparison asset or backtest curve"""
+        return self.__performance
+
+    @performance.setter
+    def performance(self, value: Tuple[FieldValueMap, ...]):
+        self._property_changed('performance')
+        self.__performance = value        
+
     @property
     def id(self) -> str:
         """Marquee unique identifier"""
@@ -1361,6 +1397,7 @@ class BacktestResult(Base):
         self,
         backtest_id: str = None,
         performance: Tuple[FieldValueMap, ...] = None,
+        portfolio: Tuple[FieldValueMap, ...] = None,
         stats: PerformanceStats = None,
         performance_decompositions: Tuple[BacktestPerformanceDecomposition, ...] = None,
         risks: Tuple[BacktestRisk, ...] = None,
@@ -1373,6 +1410,7 @@ class BacktestResult(Base):
         super().__init__()
         self.backtest_id = backtest_id
         self.performance = performance
+        self.portfolio = portfolio
         self.stats = stats
         self.performance_decompositions = performance_decompositions
         self.risks = risks
@@ -1402,6 +1440,16 @@ class BacktestResult(Base):
         self._property_changed('performance')
         self.__performance = value        
 
+    @property
+    def portfolio(self) -> Tuple[FieldValueMap, ...]:
+        """Backtest entry/exit transactions and portfolio composition."""
+        return self.__portfolio
+
+    @portfolio.setter
+    def portfolio(self, value: Tuple[FieldValueMap, ...]):
+        self._property_changed('portfolio')
+        self.__portfolio = value        
+
     @property
     def stats(self) -> PerformanceStats:
         """Performance statistics."""
