@@ -212,6 +212,55 @@ class ChartShare(Base):
         self.__version = value        
 
 
+class ChartTime(Base):
+        
+    """An object which contains all the time settings."""
+
+    @camel_case_translate
+    def __init__(
+        self,
+        start: str = None,
+        end: str = None,
+        timezone: str = None,
+        name: str = None
+    ):        
+        super().__init__()
+        self.start = start
+        self.end = end
+        self.timezone = timezone
+        self.name = name
+
+    @property
+    def start(self) -> str:
+        """Start time in hh:mm:ss format"""
+        return self.__start
+
+    @start.setter
+    def start(self, value: str):
+        self._property_changed('start')
+        self.__start = value        
+
+    @property
+    def end(self) -> str:
+        """End time in hh:mm:ss format"""
+        return self.__end
+
+    @end.setter
+    def end(self, value: str):
+        self._property_changed('end')
+        self.__end = value        
+
+    @property
+    def timezone(self) -> str:
+        """The timezone to use"""
+        return self.__timezone
+
+    @timezone.setter
+    def timezone(self, value: str):
+        self._property_changed('timezone')
+        self.__timezone = value        
+
+
 class XAxisSettings(Base):
         
     """An object which contains all the settings for the X axis"""
@@ -219,15 +268,26 @@ class XAxisSettings(Base):
     @camel_case_translate
     def __init__(
         self,
+        auto_fit_range_to_data: bool = None,
         show_grid_lines: bool = None,
         x_axis_date_format: str = None,
         name: str = None
     ):        
         super().__init__()
+        self.auto_fit_range_to_data = auto_fit_range_to_data
         self.show_grid_lines = show_grid_lines
         self.x_axis_date_format = x_axis_date_format
         self.name = name
 
+    @property
+    def auto_fit_range_to_data(self) -> bool:
+        return self.__auto_fit_range_to_data
+
+    @auto_fit_range_to_data.setter
+    def auto_fit_range_to_data(self, value: bool):
+        self._property_changed('auto_fit_range_to_data')
+        self.__auto_fit_range_to_data = value        
+
     @property
     def show_grid_lines(self) -> bool:
         return self.__show_grid_lines
@@ -257,6 +317,9 @@ class YAxisSettings(Base):
         decimal_precision: int = None,
         id_: str = None,
         label: str = None,
+        label_format: str = None,
+        max_: int = None,
+        min_: int = None,
         show_grid_lines: bool = None,
         name: str = None
     ):        
@@ -264,6 +327,9 @@ class YAxisSettings(Base):
         self.decimal_precision = decimal_precision
         self.__id = id_
         self.label = label
+        self.label_format = label_format
+        self.__max = max_
+        self.__min = min_
         self.show_grid_lines = show_grid_lines
         self.name = name
 
@@ -297,6 +363,36 @@ class YAxisSettings(Base):
         self._property_changed('label')
         self.__label = value        
 
+    @property
+    def label_format(self) -> str:
+        """The format to apply to the Y axis ticks."""
+        return self.__label_format
+
+    @label_format.setter
+    def label_format(self, value: str):
+        self._property_changed('label_format')
+        self.__label_format = value        
+
+    @property
+    def max(self) -> int:
+        """Maximum Y axis tick value."""
+        return self.__max
+
+    @max.setter
+    def max(self, value: int):
+        self._property_changed('max')
+        self.__max = value        
+
+    @property
+    def min(self) -> int:
+        """Minimum Y axis tick value."""
+        return self.__min
+
+    @min.setter
+    def min(self, value: int):
+        self._property_changed('min')
+        self.__min = value        
+
     @property
     def show_grid_lines(self) -> bool:
         """Whether or not the grid lines for this axis will be rendered on the chart."""
@@ -632,6 +728,7 @@ class Chart(Base):
         chart_type: Union[ChartType, str] = None,
         chart_properties: Tuple[ChartProperties, ...] = None,
         regression_properties: Tuple[ChartRegression, ...] = None,
+        interval: str = None,
         relative_start_date: str = None,
         relative_end_date: str = None,
         start_date: datetime.date = None,
@@ -639,10 +736,12 @@ class Chart(Base):
         start_time: datetime.datetime = None,
         end_time: datetime.datetime = None,
         tags: Tuple[str, ...] = None,
+        auto_tags: Tuple[str, ...] = None,
         show_statistics: bool = None,
         copy_from_id: str = None,
         version: int = None,
         draft_view_id: str = None,
+        time_settings: ChartTime = None,
         x_axis_settings: XAxisSettings = None,
         y_axes_settings: Tuple[YAxisSettings, ...] = None
     ):        
@@ -662,6 +761,7 @@ class Chart(Base):
         self.chart_type = chart_type
         self.chart_properties = chart_properties
         self.regression_properties = regression_properties
+        self.interval = interval
         self.relative_start_date = relative_start_date
         self.relative_end_date = relative_end_date
         self.start_date = start_date
@@ -669,10 +769,12 @@ class Chart(Base):
         self.start_time = start_time
         self.end_time = end_time
         self.tags = tags
+        self.auto_tags = auto_tags
         self.show_statistics = show_statistics
         self.copy_from_id = copy_from_id
         self.version = version
         self.draft_view_id = draft_view_id
+        self.time_settings = time_settings
         self.x_axis_settings = x_axis_settings
         self.y_axes_settings = y_axes_settings
 
@@ -825,6 +927,16 @@ class Chart(Base):
         self._property_changed('regression_properties')
         self.__regression_properties = value        
 
+    @property
+    def interval(self) -> str:
+        """The interval value to be used e.g. Daily, 1M or 20Y"""
+        return self.__interval
+
+    @interval.setter
+    def interval(self, value: str):
+        self._property_changed('interval')
+        self.__interval = value        
+
     @property
     def relative_start_date(self) -> str:
         return self.__relative_start_date
@@ -890,6 +1002,17 @@ class Chart(Base):
         self._property_changed('tags')
         self.__tags = value        
 
+    @property
+    def auto_tags(self) -> Tuple[str, ...]:
+        """Metadata associated with the object. Provide an array of strings which will be
+           indexed for search and locating related objects"""
+        return self.__auto_tags
+
+    @auto_tags.setter
+    def auto_tags(self, value: Tuple[str, ...]):
+        self._property_changed('auto_tags')
+        self.__auto_tags = value        
+
     @property
     def show_statistics(self) -> bool:
         """Flag to show the statistic in the chart area"""
@@ -930,6 +1053,16 @@ class Chart(Base):
         self._property_changed('draft_view_id')
         self.__draft_view_id = value        
 
+    @property
+    def time_settings(self) -> ChartTime:
+        """Start / end time settings with timezone"""
+        return self.__time_settings
+
+    @time_settings.setter
+    def time_settings(self, value: ChartTime):
+        self._property_changed('time_settings')
+        self.__time_settings = value        
+
     @property
     def x_axis_settings(self) -> XAxisSettings:
         """Settings for the X axis"""
