@@ -31,12 +31,18 @@ class ComponentType(EnumBase, Enum):
     barChart = 'barChart'
     commentary = 'commentary'
     commentaryPromo = 'commentaryPromo'
+    container = 'container'
+    legend = 'legend'
+    market = 'market'
     monitor = 'monitor'
     plot = 'plot'
     promo = 'promo'
     rates = 'rates'
+    relatedLinks = 'relatedLinks'
     research = 'research'
+    selector = 'selector'
     separator = 'separator'
+    stackedBarChart = 'stackedBarChart'
     
     def __repr__(self):
         return self.value
@@ -329,7 +335,7 @@ class CommentaryPromoComponentParameters(Base):
     @camel_case_translate
     def __init__(
         self,
-        height: float,
+        height: float = None,
         tooltip: str = None,
         commentary_channels: Tuple[str, ...] = None,
         transparent: bool = None,
@@ -395,60 +401,118 @@ class CommentaryPromoComponentParameters(Base):
         self.__body = value        
 
 
-class Entitlements(Base):
+class ContainerComponentParameters(Base):
         
-    """Defines the entitlements of a given resource"""
+    """Parameters provided for a container component"""
 
     @camel_case_translate
     def __init__(
         self,
-        view: Tuple[str, ...],
-        edit: Tuple[str, ...],
-        admin: Tuple[str, ...],
+        component_id: str = None,
         name: str = None
     ):        
         super().__init__()
-        self.view = view
-        self.edit = edit
-        self.admin = admin
+        self.component_id = component_id
         self.name = name
 
     @property
-    def view(self) -> Tuple[str, ...]:
-        """Permission to view the resource and its contents"""
-        return self.__view
+    def component_id(self) -> str:
+        """The id of the component, that the container should render initially."""
+        return self.__component_id
 
-    @view.setter
-    def view(self, value: Tuple[str, ...]):
-        self._property_changed('view')
-        self.__view = value        
+    @component_id.setter
+    def component_id(self, value: str):
+        self._property_changed('component_id')
+        self.__component_id = value        
+
+
+class LegendItem(Base):
+        
+    """Parameters provided for a legend item"""
+
+    @camel_case_translate
+    def __init__(
+        self,
+        color: str,
+        icon: str,
+        name: str,
+        tooltip: str = None
+    ):        
+        super().__init__()
+        self.color = color
+        self.icon = icon
+        self.name = name
+        self.tooltip = tooltip
 
     @property
-    def edit(self) -> Tuple[str, ...]:
-        """Permission to edit details about the resource content, excluding entitlements.
-           Can also delete the resource"""
-        return self.__edit
+    def color(self) -> str:
+        """Hex color of the legend item. i.e. #FF0000"""
+        return self.__color
 
-    @edit.setter
-    def edit(self, value: Tuple[str, ...]):
-        self._property_changed('edit')
-        self.__edit = value        
+    @color.setter
+    def color(self, value: str):
+        self._property_changed('color')
+        self.__color = value        
 
     @property
-    def admin(self) -> Tuple[str, ...]:
-        """Permission to edit all details of the resource, including entitlements. Can also
-           delete the resource"""
-        return self.__admin
+    def icon(self) -> str:
+        """Icon of the legend. i.e. Circle or Square."""
+        return self.__icon
 
-    @admin.setter
-    def admin(self, value: Tuple[str, ...]):
-        self._property_changed('admin')
-        self.__admin = value        
+    @icon.setter
+    def icon(self, value: str):
+        self._property_changed('icon')
+        self.__icon = value        
+
+    @property
+    def name(self) -> str:
+        """Name of the legend item."""
+        return self.__name
+
+    @name.setter
+    def name(self, value: str):
+        self._property_changed('name')
+        self.__name = value        
+
+    @property
+    def tooltip(self) -> str:
+        """Tooltip for the legend item."""
+        return self.__tooltip
+
+    @tooltip.setter
+    def tooltip(self, value: str):
+        self._property_changed('tooltip')
+        self.__tooltip = value        
+
+
+class MarketComponentParameters(Base):
+        
+    """Parameters provided for a market component."""
+
+    @camel_case_translate
+    def __init__(
+        self,
+        height: float,
+        name: str = None
+    ):        
+        super().__init__()
+        self.height = height
+        self.name = name
+
+    @property
+    def height(self) -> float:
+        """Used for restricting the height in pixels of the component on a workspace"""
+        return self.__height
+
+    @height.setter
+    def height(self, value: float):
+        self._property_changed('height')
+        self.__height = value        
 
 
 class MonitorComponentParameters(Base):
         
-    """Parameters provided for a monitor component"""
+    """Parameters provided for a monitor component."""
 
     @camel_case_translate
     def __init__(
@@ -554,12 +618,16 @@ class PromoComponentParameters(Base):
         height: float,
         transparent: bool = None,
         body: str = None,
+        size: str = None,
+        hide_border: bool = None,
         name: str = None
     ):        
         super().__init__()
         self.height = height
         self.transparent = transparent
         self.body = body
+        self.size = size
+        self.hide_border = hide_border
         self.name = name
 
     @property
@@ -592,6 +660,26 @@ class PromoComponentParameters(Base):
         self._property_changed('body')
         self.__body = value        
 
+    @property
+    def size(self) -> str:
+        """Represents the size of the text for the promo component."""
+        return self.__size
+
+    @size.setter
+    def size(self, value: str):
+        self._property_changed('size')
+        self.__size = value        
+
+    @property
+    def hide_border(self) -> bool:
+        """Whether or not to hide the border of the card."""
+        return self.__hide_border
+
+    @hide_border.setter
+    def hide_border(self, value: bool):
+        self._property_changed('hide_border')
+        self.__hide_border = value        
+
 
 class RatesComponentParameters(Base):
         
@@ -630,6 +718,65 @@ class RatesComponentParameters(Base):
         self.__tooltip = value        
 
 
+class RelatedLink(Base):
+        
+    """Parameters provided for a related link"""
+
+    @camel_case_translate
+    def __init__(
+        self,
+        type_: str,
+        name: str,
+        link: str,
+        description: str = None
+    ):        
+        super().__init__()
+        self.__type = type_
+        self.name = name
+        self.description = description
+        self.link = link
+
+    @property
+    def type(self) -> str:
+        """Type of related link eg. internal or external"""
+        return self.__type
+
+    @type.setter
+    def type(self, value: str):
+        self._property_changed('type')
+        self.__type = value        
+
+    @property
+    def name(self) -> str:
+        """Name to be displayed."""
+        return self.__name
+
+    @name.setter
+    def name(self, value: str):
+        self._property_changed('name')
+        self.__name = value        
+
+    @property
+    def description(self) -> str:
+        """Description of related link."""
+        return self.__description
+
+    @description.setter
+    def description(self, value: str):
+        self._property_changed('description')
+        self.__description = value        
+
+    @property
+    def link(self) -> str:
+        """URL of related link."""
+        return self.__link
+
+    @link.setter
+    def link(self, value: str):
+        self._property_changed('link')
+        self.__link = value        
+
+
 class ResearchComponentParameters(Base):
         
     """Parameters provided for a research component"""
@@ -691,6 +838,55 @@ class ResearchComponentParameters(Base):
         self.__commentary_to_desktop_link = value        
 
 
+class SelectorComponentOption(Base):
+        
+    """A selector component option."""
+
+    @camel_case_translate
+    def __init__(
+        self,
+        id_: str,
+        name: str,
+        tags: Tuple[str, ...]
+    ):        
+        super().__init__()
+        self.__id = id_
+        self.name = name
+        self.tags = tags
+
+    @property
+    def id(self) -> str:
+        """A unique id of the selector option."""
+        return self.__id
+
+    @id.setter
+    def id(self, value: str):
+        self._property_changed('id')
+        self.__id = value        
+
+    @property
+    def name(self) -> str:
+        """Name of the option in the dropdown. This will be the text displayed in the
+           dropdown."""
+        return self.__name
+
+    @name.setter
+    def name(self, value: str):
+        self._property_changed('name')
+        self.__name = value        
+
+    @property
+    def tags(self) -> Tuple[str, ...]:
+        """An array of component tags for this selector option. This should match in length
+           the selector component containerIds length."""
+        return self.__tags
+
+    @tags.setter
+    def tags(self, value: Tuple[str, ...]):
+        self._property_changed('tags')
+        self.__tags = value        
+
+
 class SeparatorComponentParameters(Base):
         
     """Parameters provided for a separator component"""
@@ -699,11 +895,15 @@ class SeparatorComponentParameters(Base):
     def __init__(
         self,
         height: float,
-        name: str = None
+        name: str = None,
+        size: str = None,
+        show_more_url: str = None
     ):        
         super().__init__()
         self.height = height
         self.name = name
+        self.size = size
+        self.show_more_url = show_more_url
 
     @property
     def height(self) -> float:
@@ -725,6 +925,26 @@ class SeparatorComponentParameters(Base):
         self._property_changed('name')
         self.__name = value        
 
+    @property
+    def size(self) -> str:
+        """Represents the size of the text for the separator component."""
+        return self.__size
+
+    @size.setter
+    def size(self, value: str):
+        self._property_changed('size')
+        self.__size = value        
+
+    @property
+    def show_more_url(self) -> str:
+        """Show more link on the component header. Provide absolute or relative url."""
+        return self.__show_more_url
+
+    @show_more_url.setter
+    def show_more_url(self, value: str):
+        self._property_changed('show_more_url')
+        self.__show_more_url = value        
+
 
 class WorkspaceDate(Base):
         
@@ -799,6 +1019,216 @@ class WorkspaceTab(Base):
         self.__name = value        
 
 
+class LegendComponentParameters(Base):
+        
+    """Parameters provided for the legend component."""
+
+    @camel_case_translate
+    def __init__(
+        self,
+        height: float,
+        items: Tuple[LegendItem, ...],
+        position: str = None,
+        transparent: bool = None,
+        name: str = None
+    ):        
+        super().__init__()
+        self.height = height
+        self.items = items
+        self.position = position
+        self.transparent = transparent
+        self.name = name
+
+    @property
+    def height(self) -> float:
+        """Used for restricting the height in pixels of the component on a workspace."""
+        return self.__height
+
+    @height.setter
+    def height(self, value: float):
+        self._property_changed('height')
+        self.__height = value        
+
+    @property
+    def items(self) -> Tuple[LegendItem, ...]:
+        """The legend items displayed in the component."""
+        return self.__items
+
+    @items.setter
+    def items(self, value: Tuple[LegendItem, ...]):
+        self._property_changed('items')
+        self.__items = value        
+
+    @property
+    def position(self) -> str:
+        """Whether or not to position the legend items on the left or right."""
+        return self.__position
+
+    @position.setter
+    def position(self, value: str):
+        self._property_changed('position')
+        self.__position = value        
+
+    @property
+    def transparent(self) -> bool:
+        """Sets the component card to have a transparent background."""
+        return self.__transparent
+
+    @transparent.setter
+    def transparent(self, value: bool):
+        self._property_changed('transparent')
+        self.__transparent = value        
+
+
+class RelatedLinksComponentParameters(Base):
+        
+    """Parameters provided for a related link components"""
+
+    @camel_case_translate
+    def __init__(
+        self,
+        height: float,
+        links: Tuple[RelatedLink, ...],
+        title: str,
+        name: str = None
+    ):        
+        super().__init__()
+        self.height = height
+        self.links = links
+        self.title = title
+        self.name = name
+
+    @property
+    def height(self) -> float:
+        """Used for restricting the height in pixels of the component on a workspace"""
+        return self.__height
+
+    @height.setter
+    def height(self, value: float):
+        self._property_changed('height')
+        self.__height = value        
+
+    @property
+    def links(self) -> Tuple[RelatedLink, ...]:
+        """Description associated with link"""
+        return self.__links
+
+    @links.setter
+    def links(self, value: Tuple[RelatedLink, ...]):
+        self._property_changed('links')
+        self.__links = value        
+
+    @property
+    def title(self) -> str:
+        """Title of the related link component."""
+        return self.__title
+
+    @title.setter
+    def title(self, value: str):
+        self._property_changed('title')
+        self.__title = value        
+
+
+class SelectorComponentParameters(Base):
+        
+    """Parameters provided for a selector component"""
+
+    @camel_case_translate
+    def __init__(
+        self,
+        container_ids: Tuple[str, ...],
+        height: float,
+        options: Tuple[SelectorComponentOption, ...],
+        default_option_index: float = None,
+        title: str = None,
+        tooltip: str = None,
+        width: float = None,
+        name: str = None
+    ):        
+        super().__init__()
+        self.container_ids = container_ids
+        self.default_option_index = default_option_index
+        self.height = height
+        self.options = options
+        self.title = title
+        self.tooltip = tooltip
+        self.width = width
+        self.name = name
+
+    @property
+    def container_ids(self) -> Tuple[str, ...]:
+        """The component ids of the containers the selector will fill using it's options."""
+        return self.__container_ids
+
+    @container_ids.setter
+    def container_ids(self, value: Tuple[str, ...]):
+        self._property_changed('container_ids')
+        self.__container_ids = value        
+
+    @property
+    def default_option_index(self) -> float:
+        """The default option for the selector. This references the index in the options
+           array. Defaults to 0."""
+        return self.__default_option_index
+
+    @default_option_index.setter
+    def default_option_index(self, value: float):
+        self._property_changed('default_option_index')
+        self.__default_option_index = value        
+
+    @property
+    def height(self) -> float:
+        """Used for restricting the height in pixels of the component on a workspace"""
+        return self.__height
+
+    @height.setter
+    def height(self, value: float):
+        self._property_changed('height')
+        self.__height = value        
+
+    @property
+    def options(self) -> Tuple[SelectorComponentOption, ...]:
+        """A list of options for the selector dropdown. Options references other components
+           in the workspace"""
+        return self.__options
+
+    @options.setter
+    def options(self, value: Tuple[SelectorComponentOption, ...]):
+        self._property_changed('options')
+        self.__options = value        
+
+    @property
+    def title(self) -> str:
+        """Title of the text to the left of the selector component."""
+        return self.__title
+
+    @title.setter
+    def title(self, value: str):
+        self._property_changed('title')
+        self.__title = value        
+
+    @property
+    def tooltip(self) -> str:
+        """Tooltip for the title to the left of the selector component."""
+        return self.__tooltip
+
+    @tooltip.setter
+    def tooltip(self, value: str):
+        self._property_changed('tooltip')
+        self.__tooltip = value        
+
+    @property
+    def width(self) -> float:
+        """Used for restricting the width in pixels of the component on a workspace.
+           Defaults to 280px."""
+        return self.__width
+
+    @width.setter
+    def width(self, value: float):
+        self._property_changed('width')
+        self.__width = value        
+
+
 class WorkspaceComponent(Base):
         
     """Parameters provided for a market workspace"""
@@ -808,11 +1238,15 @@ class WorkspaceComponent(Base):
         self,
         id_: dict,
         type_: Union[ComponentType, str],
+        hide: bool = None,
+        tags: Tuple[str, ...] = None,
         parameters: dict = None,
         name: str = None
     ):        
         super().__init__()
         self.__id = id_
+        self.hide = hide
+        self.tags = tags
         self.__type = get_enum_value(ComponentType, type_)
         self.parameters = parameters
         self.name = name
@@ -826,6 +1260,27 @@ class WorkspaceComponent(Base):
         self._property_changed('id')
         self.__id = value        
 
+    @property
+    def hide(self) -> bool:
+        """Whether to hide the component."""
+        return self.__hide
+
+    @hide.setter
+    def hide(self, value: bool):
+        self._property_changed('hide')
+        self.__hide = value        
+
+    @property
+    def tags(self) -> Tuple[str, ...]:
+        """An array of component tags for this component. Tags are referenced by other
+           components within the workspace."""
+        return self.__tags
+
+    @tags.setter
+    def tags(self, value: Tuple[str, ...]):
+        self._property_changed('tags')
+        self.__tags = value        
+
     @property
     def type(self) -> Union[ComponentType, str]:
         """Enum listing supported component types"""
@@ -855,28 +1310,32 @@ class WorkspaceParameters(Base):
         self,
         layout: str,
         components: Tuple[WorkspaceComponent, ...],
-        tabs: Tuple[WorkspaceTab, ...] = None,
-        disclaimer: str = None,
+        can_share: bool = None,
         date: WorkspaceDate = None,
+        disclaimer: str = None,
+        maintainers: Tuple[str, ...] = None,
+        tabs: Tuple[WorkspaceTab, ...] = None,
         name: str = None
     ):        
         super().__init__()
-        self.layout = layout
+        self.can_share = can_share
         self.components = components
-        self.tabs = tabs
-        self.disclaimer = disclaimer
         self.date = date
+        self.disclaimer = disclaimer
+        self.layout = layout
+        self.maintainers = maintainers
+        self.tabs = tabs
         self.name = name
 
     @property
-    def layout(self) -> str:
-        """A workspace layout expression. For example: r(c12($0))"""
-        return self.__layout
+    def can_share(self) -> bool:
+        """ whether the user has the ability to share a cashboard"""
+        return self.__can_share
 
-    @layout.setter
-    def layout(self, value: str):
-        self._property_changed('layout')
-        self.__layout = value        
+    @can_share.setter
+    def can_share(self, value: bool):
+        self._property_changed('can_share')
+        self.__can_share = value        
 
     @property
     def components(self) -> Tuple[WorkspaceComponent, ...]:
@@ -889,14 +1348,14 @@ class WorkspaceParameters(Base):
         self.__components = value        
 
     @property
-    def tabs(self) -> Tuple[WorkspaceTab, ...]:
-        """Tabs of additional market workspaces that can be linked to."""
-        return self.__tabs
+    def date(self) -> WorkspaceDate:
+        """Relative date to display at the bottom of the page."""
+        return self.__date
 
-    @tabs.setter
-    def tabs(self, value: Tuple[WorkspaceTab, ...]):
-        self._property_changed('tabs')
-        self.__tabs = value        
+    @date.setter
+    def date(self, value: WorkspaceDate):
+        self._property_changed('date')
+        self.__date = value        
 
     @property
     def disclaimer(self) -> str:
@@ -909,14 +1368,34 @@ class WorkspaceParameters(Base):
         self.__disclaimer = value        
 
     @property
-    def date(self) -> WorkspaceDate:
-        """Relative date to display at the bottom of the page."""
-        return self.__date
+    def layout(self) -> str:
+        """A workspace layout expression. For example: r(c12($0))"""
+        return self.__layout
 
-    @date.setter
-    def date(self, value: WorkspaceDate):
-        self._property_changed('date')
-        self.__date = value        
+    @layout.setter
+    def layout(self, value: str):
+        self._property_changed('layout')
+        self.__layout = value        
+
+    @property
+    def maintainers(self) -> Tuple[str, ...]:
+        """User Ids of the markets workspace maintainers."""
+        return self.__maintainers
+
+    @maintainers.setter
+    def maintainers(self, value: Tuple[str, ...]):
+        self._property_changed('maintainers')
+        self.__maintainers = value        
+
+    @property
+    def tabs(self) -> Tuple[WorkspaceTab, ...]:
+        """Tabs of additional market workspaces that can be linked to."""
+        return self.__tabs
+
+    @tabs.setter
+    def tabs(self, value: Tuple[WorkspaceTab, ...]):
+        self._property_changed('tabs')
+        self.__tabs = value        
 
 
 class Workspace(Base):
@@ -940,7 +1419,7 @@ class Workspace(Base):
         entitlements: Entitlements = None,
         folder_name: str = None,
         description: str = None,
-        children_ids: Tuple[str, ...] = None
+        children_aliases: Tuple[str, ...] = None
     ):        
         super().__init__()
         self.__id = id_
@@ -957,7 +1436,7 @@ class Workspace(Base):
         self.entitlements = entitlements
         self.folder_name = folder_name
         self.description = description
-        self.children_ids = children_ids
+        self.children_aliases = children_aliases
 
     @property
     def id(self) -> str:
@@ -1100,11 +1579,11 @@ class Workspace(Base):
         self.__description = value        
 
     @property
-    def children_ids(self) -> Tuple[str, ...]:
+    def children_aliases(self) -> Tuple[str, ...]:
         """Child workspaces for navigation"""
-        return self.__children_ids
+        return self.__children_aliases
 
-    @children_ids.setter
-    def children_ids(self, value: Tuple[str, ...]):
-        self._property_changed('children_ids')
-        self.__children_ids = value        
+    @children_aliases.setter
+    def children_aliases(self, value: Tuple[str, ...]):
+        self._property_changed('children_aliases')
+        self.__children_aliases = value        
