@@ -24,7 +24,9 @@ from gs_quant.context_base import ContextMeta
 from gs_quant.errors import MqValueError
 from gs_quant.markets import MarketDataCoordinate
 from gs_quant.session import GsSession, Environment
-from gs_quant.target.data import MDAPIDataQuery, MarketDataVendor, DataSetEntity
+from gs_quant.target.common import FieldFilterMap
+from gs_quant.target.coordinates import MDAPIDataQuery
+from gs_quant.target.data import MarketDataVendor, DataSetEntity, DataQuery
 
 test_coordinates = (
     MarketDataCoordinate(mkt_type='Prime', mkt_quoting_style='price', mkt_asset='335320934'),
@@ -235,6 +237,7 @@ def test_coordinate_last(mocker):
 
     expected_result = pd.DataFrame(
         data={
+            'time': ['2019-01-20T01:08:00Z', '2019-01-20T01:09:45Z'],
             'mktType': ['Prime', 'IR'],
             'mktAsset': ['335320934', 'USD'],
             'mktClass': [None, 'Swap'],
@@ -292,16 +295,16 @@ def test_get_many_defns_api(mocker):
 
 def test_coordinates_converter():
     coord = GsDataApi._coordinate_from_str("A_B_C_D")
-    assert str(coord) == 'A|B|C|D|'
+    assert str(coord) == 'A_B_C_D'
 
     coord = GsDataApi._coordinate_from_str("A_B_C.E")
-    assert str(coord) == 'A|B|C||E'
+    assert str(coord) == 'A_B_C.E'
 
     coord = GsDataApi._coordinate_from_str("A_B_.E")
-    assert str(coord) == 'A|B|||E'
+    assert str(coord) == 'A_B_.E'
 
-    coord = GsDataApi._coordinate_from_str("A_B_C_D_E.F")
-    assert str(coord) == 'A|B|C|D_E|F'
+    coord = GsDataApi._coordinate_from_str("A_B_C_D;E.F")
+    assert str(coord) == 'A_B_C_D;E.F'
 
     with pytest.raises(MqValueError, match='invalid coordinate A'):
         GsDataApi._coordinate_from_str("A")
@@ -325,5 +328,31 @@ def test_get_many_coordinates(mocker):
     assert response == ('A_B_C_D_E.F1', 'A_B_C_D_E.F2')
 
 
+def test_auto_scroll_on_pages(mocker):
+    response = {
+        "requestId": "049de678-1480000",
+        "totalPages": 5,
+        "data": [
+            {
+                "date": "2012-01-25",
+                "assetId": "MADXKSGX6921CFNF",
+                "value": 1
+            }
+        ]
+    }
+    mocker.patch.object(ContextMeta, 'current', return_value=GsSession(Environment.QA))
+    mocker.patch.object(ContextMeta.current, '_post', return_value=response)
+
+    query = DataQuery(
+        start_date=dt.date(2017, 1, 15),
+        end_date=dt.date(2017, 1, 18),
+        where=FieldFilterMap(
+            currency="GBP"
+        )
+    )
+    response = GsDataApi.get_results("test", response, query)
+    assert len(response) == 5
+
+
 if __name__ == "__main__":
     pytest.main(args=["test_data.py"])
