@@ -16,7 +16,7 @@ under the License.
 from datetime import date
 
 import pytest
-from pandas.util.testing import assert_series_equal
+from pandas.testing import assert_series_equal
 from scipy.integrate import odeint
 from gs_quant.timeseries import *
 
@@ -62,6 +62,14 @@ def test_min():
     expected = pd.Series([3.0, 2.0, 2.0, 1.0, 1.0, 2.0], index=dates)
     assert_series_equal(result, expected, obj="Minimum of multiple series")
 
+    result = min_(x, "2d")
+    expected = pd.Series([2.0, 1.0, 3.0, 3.0], index=dates[2:])
+    assert_series_equal(result, expected, obj="Minimum with strdate window")
+
+    result = min_(x, "1d")
+    expected = pd.Series([2.0, 3.0, 1.0, 3.0, 6.0], index=dates[1:])
+    assert_series_equal(result, expected, obj="Minimum with strdate window 2")
+
 
 def test_max():
     dates = [
@@ -96,6 +104,12 @@ def test_max():
     expected = pd.Series([4.0, 4.0, 4.0, 4.0, 3.0, 6.0], index=dates)
     assert_series_equal(result, expected, obj="Maximum of multiple series")
 
+    s = pd.Series([-3.0, -2.0, 3.0, -1.0, -3.0, 6.0], index=dates)
+    t = pd.Series([0, 0], index=dates[0:2])
+    result = max_([s, t], 1)
+    expected = pd.Series([0.0, 3, 0, 0, 6], index=dates[1:])
+    assert_series_equal(result, expected, obj="Maximum with constant")
+
 
 def test_range():
     dates = [
@@ -299,7 +313,6 @@ def test_std():
 
 
 def test_exponential_std():
-
     def exp_std_calc(ts, alpha=0.75):
         std = ts * 0
         for i in range(1, len(ts)):
@@ -385,6 +398,9 @@ def test_cov():
 
 
 def test_zscores():
+    with pytest.raises(MqValueError):
+        zscores(pd.Series(range(5)), "2d")
+
     assert_series_equal(zscores(pd.Series()), pd.Series())
     assert_series_equal(zscores(pd.Series(), 1), pd.Series())
 
@@ -417,6 +433,14 @@ def test_zscores():
     expected = pd.Series([0.0, -0.707106, 0.577350, -1.305582, 0.670820, 1.603567], index=dates)
     assert_series_equal(result, expected, obj="z-score window 1w", check_less_precise=True)
 
+    result = zscores(x, '1w')
+    expected = pd.Series([1.603567], index=dates[-1:])
+    assert_series_equal(result, expected, obj='z-score window string 1w', check_less_precise=True)
+
+    result = zscores(x, '1m')
+    expected = pd.Series()
+    assert_series_equal(result, expected, obj="z-score window too large", check_less_precise=True)
+
 
 def test_winsorize():
     assert_series_equal(winsorize(pd.Series()), pd.Series())
@@ -507,6 +531,8 @@ def test_percentile():
         percentile(pd.Series(), -1)
     with pytest.raises(MqError):
         percentile(pd.Series(), 100.1)
+    with pytest.raises(MqTypeError):
+        percentile(pd.Series(range(5), index=range(5)), 90, "2d")
 
     for n in range(0, 101, 5):
         assert percentile(pd.Series(x * 10 for x in range(0, 11)), n) == n
@@ -514,12 +540,27 @@ def test_percentile():
     x = percentile(pd.Series(x for x in range(0, 5)), 50, 2)
     assert_series_equal(x, pd.Series([1.5, 2.5, 3.5], index=pd.RangeIndex(2, 5)))
 
+    x = percentile(pd.Series(), 90, "1d")
+    assert_series_equal(x, pd.Series(), obj="Percentile with empty series")
+
+
+def test_percentile_str():
+    today = datetime.datetime.now()
+    days = pd.date_range(today, periods=12, freq='D')
+    start = pd.Series([29, 56, 82, 13, 35, 53, 25, 23, 21, 12, 15, 9], index=days)
+    actual = percentile(start, 2, '10d')
+    expected = pd.Series([12.18, 9.54], index=pd.date_range(today + datetime.timedelta(days=10), periods=2, freq='D'))
+    assert_series_equal(actual, expected)
+
+    actual = percentile(start, 50, '1w')
+    expected = percentile(start, 50, 7)
+    assert_series_equal(actual, expected)
+
 
 def test_regression():
-    dates = pd.date_range('2019-1-1', periods=6)
-    x1 = pd.Series([0.0, 1.0, 4.0, 9.0, 16.0, 25.0], index=dates)
-    x2 = pd.Series([0.0, 1.0, 2.0, 3.0, 4.0, 5.0], index=dates)
-    y = pd.Series([10.0, 14.0, 20.0, 28.0, 38.0, 50.0], index=dates)
+    x1 = pd.Series([0.0, 1.0, 4.0, 9.0, 16.0, 25.0, np.nan], index=pd.date_range('2019-1-1', periods=7), name='x1')
+    x2 = pd.Series([0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0], index=pd.date_range('2019-1-1', periods=8))
+    y = pd.Series([10.0, 14.0, 20.0, 28.0, 38.0, 50.0, 60.0], index=pd.date_range('2019-1-1', periods=7))
 
     regression = LinearRegression([x1, x2], y, True)
 
@@ -527,12 +568,10 @@ def test_regression():
     np.testing.assert_almost_equal(regression.coefficient(1), 1.0)
     np.testing.assert_almost_equal(regression.coefficient(2), 3.0)
 
-    with pytest.raises(ValueError):
-        regression.coefficient(3)
-
     np.testing.assert_almost_equal(regression.r_squared(), 1.0)
 
-    assert_series_equal(regression.fitted_values(), y)
+    expected = pd.Series([10.0, 14.0, 20.0, 28.0, 38.0, 50.0], index=pd.date_range('2019-1-1', periods=6))
+    assert_series_equal(regression.fitted_values(), expected)
 
     dates_predict = [date(2019, 2, 1), date(2019, 2, 2)]
     predicted = regression.predict([pd.Series([2.0, 3.0], index=dates_predict),
@@ -540,6 +579,37 @@ def test_regression():
     expected = pd.Series([30.0, 34.0], index=dates_predict)
     assert_series_equal(predicted, expected)
 
+    np.testing.assert_almost_equal(regression.standard_deviation_of_errors(), 0)
+
+
+def test_rolling_linear_regression():
+    x1 = pd.Series([0.0, 1.0, 4.0, 9.0, 16.0, 25.0, np.nan], index=pd.date_range('2019-1-1', periods=7), name='x1')
+    x2 = pd.Series([0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0], index=pd.date_range('2019-1-1', periods=8))
+    y = pd.Series([10.0, 14.0, 20.0, 28.0, 28.0, 40.0, 60.0], index=pd.date_range('2019-1-1', periods=7))
+
+    with pytest.raises(MqValueError):
+        RollingLinearRegression([x1, x2], y, 3, True)
+
+    regression = RollingLinearRegression([x1, x2], y, 4, True)
+
+    expected = pd.Series([np.nan, np.nan, np.nan, 10.0, 2.5, 19.0], index=pd.date_range('2019-1-1', periods=6))
+    assert_series_equal(regression.coefficient(0), expected, check_names=False)
+
+    expected = pd.Series([np.nan, np.nan, np.nan, 1.0, -1.5, 1.0], index=pd.date_range('2019-1-1', periods=6))
+    assert_series_equal(regression.coefficient(1), expected, check_names=False)
+
+    expected = pd.Series([np.nan, np.nan, np.nan, 3.0, 12.5, -1.0], index=pd.date_range('2019-1-1', periods=6))
+    assert_series_equal(regression.coefficient(2), expected, check_names=False)
+
+    expected = pd.Series([np.nan, np.nan, np.nan, 1.0, 0.964029, 0.901961], index=pd.date_range('2019-1-1', periods=6))
+    assert_series_equal(regression.r_squared(), expected, check_names=False)
+
+    expected = pd.Series([np.nan, np.nan, np.nan, 28.0, 28.5, 39.0], index=pd.date_range('2019-1-1', periods=6))
+    assert_series_equal(regression.fitted_values(), expected, check_names=False, check_less_precise=True)
+
+    expected = pd.Series([np.nan, np.nan, np.nan, 0.0, 2.236068, 4.472136], index=pd.date_range('2019-1-1', periods=6))
+    assert_series_equal(regression.standard_deviation_of_errors(), expected, check_names=False)
+
 
 def test_sir_model():
     n = 1000
