@@ -14,6 +14,8 @@
 # Chart Service will attempt to make public functions (not prefixed with _) from this module available. Such functions
 # should be fully documented: docstrings should describe parameters and the return value, and provide a 1-line
 # description. Type annotations should be provided for parameters.
+import re
+
 from gs_quant.datetime import relative_date_add
 from gs_quant.timeseries.datetime import *
 from .helper import plot_function
@@ -58,7 +60,7 @@ def first(x: pd.Series) -> pd.Series:
 @plot_function
 def last(x: pd.Series) -> pd.Series:
     """
-    Last value of series
+    Last value of series (as a series)
 
     :param x: time series
     :return: time series of last value
@@ -86,6 +88,35 @@ def last(x: pd.Series) -> pd.Series:
     return pd.Series(x[-1], x.index)
 
 
+@plot_function
+def last_value(x: pd.Series) -> Union[int, float]:
+    """
+    Last value of series (as a scalar)
+
+    :param x: time series
+    :return: last value
+
+    **Usage**
+
+    Return a scalar value :math:`X_T` where T is the last index value in the series.
+
+    **Examples**
+
+    Last value of series:
+
+    >>> series = generate_series(100)
+    >>> lv = last_value(series)
+
+    **See also**
+
+    :func:`last`
+
+    """
+    if x.empty:
+        raise MqValueError("cannot get last value of an empty series")
+    return x.iloc[-1]
+
+
 @plot_function
 def count(x: pd.Series) -> pd.Series:
     """
@@ -201,7 +232,14 @@ def lag(x: pd.Series, obs: Union[Window, int, str] = 1, mode: LagMode = LagMode.
     if isinstance(obs, str):
         end = x.index[-1]
         y = x.copy()  # avoid mutating the provided series
-        y.index = y.index + pd.DateOffset(relative_date_add(obs))
+
+        match = re.fullmatch('(\\d+)y', obs)
+        if match:
+            y.index += pd.DateOffset(years=int(match.group(1)))
+            y = y.groupby(y.index).first()
+        else:
+            y.index += pd.DateOffset(relative_date_add(obs))
+
         if mode == LagMode.EXTEND:
             return y
         return y[:end]
